from django.dispatch import Signal

# Arguments: "user"
signup_complete = Signal()

# Arguments: "user"
activation_complete = Signal()

# Arguments: "user", "old_email"
confirmation_complete = Signal()

# Arguments: "user"
password_complete = Signal()

# Arguments: "user", "prev_email", "new_email"
email_change = Signal()

# Arguments: "user"
profile_change = Signal()

# Arguments: "user"
account_signin = Signal()

# Arguments: "user"
account_signout = Signal()



from django.contrib import admin
from django.contrib.auth import get_user_model
from django.contrib.auth.admin import UserAdmin
from django.utils.translation import gettext as _
from guardian.admin import GuardedModelAdmin

from userena.models import UserenaSignup
from userena import settings as userena_settings
from userena.utils import get_profile_model


class UserenaSignupInline(admin.StackedInline):
    model = UserenaSignup
    max_num = 1


class UserenaAdmin(UserAdmin, GuardedModelAdmin):
    inlines = [UserenaSignupInline]
    list_display = (
        "username",
        "email",
        "first_name",
        "last_name",
        "is_staff",
        "is_active",
        "date_joined",
    )
    list_filter = ("is_staff", "is_superuser", "is_active")


if userena_settings.USERENA_REGISTER_USER:
    try:
        admin.site.unregister(get_user_model())
    except admin.sites.NotRegistered:
        pass

    admin.site.register(get_user_model(), UserenaAdmin)

if userena_settings.USERENA_REGISTER_PROFILE:
    admin.site.register(get_profile_model(), GuardedModelAdmin)



from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [migrations.swappable_dependency(settings.AUTH_USER_MODEL)]

    operations = [
        migrations.CreateModel(
            name="Message",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("body", models.TextField(verbose_name="body")),
                (
                    "sent_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="sent at"),
                ),
                (
                    "sender_deleted_at",
                    models.DateTimeField(
                        null=True, verbose_name="sender deleted at", blank=True
                    ),
                ),
            ],
            options={
                "ordering": ["-sent_at"],
                "verbose_name": "message",
                "verbose_name_plural": "messages",
            },
        ),
        migrations.CreateModel(
            name="MessageContact",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "latest_message",
                    models.ForeignKey(
                        verbose_name="latest message",
                        to="umessages.Message",
                        on_delete=models.CASCADE,
                    ),
                ),
                (
                    "um_from_user",
                    models.ForeignKey(
                        related_name="um_from_users",
                        verbose_name="from user",
                        to=settings.AUTH_USER_MODEL,
                        on_delete=models.CASCADE,
                    ),
                ),
                (
                    "um_to_user",
                    models.ForeignKey(
                        related_name="um_to_users",
                        verbose_name="to user",
                        to=settings.AUTH_USER_MODEL,
                        on_delete=models.CASCADE,
                    ),
                ),
            ],
            options={
                "ordering": ["latest_message"],
                "verbose_name": "contact",
                "verbose_name_plural": "contacts",
            },
        ),
        migrations.CreateModel(
            name="MessageRecipient",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "read_at",
                    models.DateTimeField(null=True, verbose_name="read at", blank=True),
                ),
                (
                    "deleted_at",
                    models.DateTimeField(
                        null=True, verbose_name="recipient deleted at", blank=True
                    ),
                ),
                (
                    "message",
                    models.ForeignKey(
                        verbose_name="message",
                        to="umessages.Message",
                        on_delete=models.CASCADE,
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        verbose_name="recipient",
                        to=settings.AUTH_USER_MODEL,
                        on_delete=models.CASCADE,
                    ),
                ),
            ],
            options={"verbose_name": "recipient", "verbose_name_plural": "recipients"},
        ),
        migrations.AddField(
            model_name="message",
            name="recipients",
            field=models.ManyToManyField(
                related_name="received_messages",
                verbose_name="recipients",
                through="umessages.MessageRecipient",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AddField(
            model_name="message",
            name="sender",
            field=models.ForeignKey(
                related_name="sent_messages",
                verbose_name="sender",
                to=settings.AUTH_USER_MODEL,
                on_delete=models.CASCADE,
            ),
        ),
        migrations.AlterUniqueTogether(
            name="messagecontact", unique_together=set([("um_from_user", "um_to_user")])
        ),
    ]



from django.contrib.auth import get_user_model
from django.test import TestCase

from userena.models import UserenaSignup
from userena import settings as userena_settings
from userena.utils import get_user_profile

from guardian.shortcuts import get_perms

import datetime, re

User = get_user_model()


class UserenaManagerTests(TestCase):
    """ Test the manager of Userena """

    user_info = {
        "username": "alice",
        "password": "swordfish",
        "email": "alice@example.com",
    }

    fixtures = ["users"]

    def test_create_inactive_user(self):
        """
        Test the creation of a new user.

        ``UserenaSignup.create_inactive_user`` should create a new user that is
        not active. The user should get an ``activation_key`` that is used to
        set the user as active.

        Every user also has a profile, so this method should create an empty
        profile.

        """
        # Check that the fields are set.
        new_user = UserenaSignup.objects.create_user(**self.user_info)
        self.assertEqual(new_user.username, self.user_info["username"])
        self.assertEqual(new_user.email, self.user_info["email"])
        self.assertTrue(new_user.check_password(self.user_info["password"]))

        # User should be inactive
        self.assertFalse(new_user.is_active)

        # User has a valid activation key
        self.assertTrue(
            re.match(r"^[\w]{40}$", new_user.userena_signup.activation_key)
        )

        # User now has an profile.
        self.assertTrue(get_user_profile(user=new_user))

        # User should be saved
        self.assertEqual(User.objects.filter(email=self.user_info["email"]).count(), 1)

    def test_activation_valid(self):
        """
        Valid activation of an user.

        Activation of an user with a valid ``activation_key`` should activate
        the user and set a new invalid ``activation_key`` that is defined in
        the setting ``USERENA_ACTIVATED``.

        """
        user = UserenaSignup.objects.create_user(**self.user_info)
        active_user = UserenaSignup.objects.activate_user(
            user.userena_signup.activation_key
        )
        profile = get_user_profile(user=active_user)

        # The returned user should be the same as the one just created.
        self.assertEqual(user, active_user)

        # The user should now be active.
        self.assertTrue(active_user.is_active)

        # The user should have permission to view and change its profile
        self.assertTrue("view_profile" in get_perms(active_user, profile))
        self.assertTrue("change_profile" in get_perms(active_user, profile))

        # The activation key should be the same as in the settings
        self.assertEqual(
            active_user.userena_signup.activation_key,
            userena_settings.USERENA_ACTIVATED,
        )

    def test_activation_invalid(self):
        """
        Activation with a key that's invalid should make
        ``UserenaSignup.objects.activate_user`` return ``False``.

        """
        # Wrong key
        self.assertFalse(UserenaSignup.objects.activate_user("wrong_key"))

        # At least the right length
        invalid_key = 10 * "a1b2"
        self.assertFalse(UserenaSignup.objects.activate_user(invalid_key))

    def test_activation_expired(self):
        """
        Activation with a key that's expired should also make
        ``UserenaSignup.objects.activation_user`` return ``False``.

        """
        user = UserenaSignup.objects.create_user(**self.user_info)

        # Set the date that the key is created a day further away than allowed
        user.date_joined -= datetime.timedelta(
            days=userena_settings.USERENA_ACTIVATION_DAYS + 1
        )
        user.save()

        # Try to activate the user
        UserenaSignup.objects.activate_user(user.userena_signup.activation_key)

        active_user = User.objects.get(username="alice")

        # UserenaSignup activation should have failed
        self.assertFalse(active_user.is_active)

        # The activation key should still be a hash
        self.assertEqual(
            user.userena_signup.activation_key,
            active_user.userena_signup.activation_key,
        )

    def test_confirmation_valid(self):
        """
        Confirmation of a new e-mail address with turns out to be valid.

        """
        new_email = "john@newexample.com"
        user = User.objects.get(pk=1)
        user.userena_signup.change_email(new_email)

        # Confirm email
        confirmed_user = UserenaSignup.objects.confirm_email(
            user.userena_signup.email_confirmation_key
        )
        self.assertEqual(user, confirmed_user)

        # Check the new email is set.
        self.assertEqual(confirmed_user.email, new_email)

        # ``email_new`` and ``email_verification_key`` should be empty
        self.assertFalse(confirmed_user.userena_signup.email_unconfirmed)
        self.assertFalse(confirmed_user.userena_signup.email_confirmation_key)

    def test_confirmation_invalid(self):
        """
        Trying to confirm a new e-mail address when the ``confirmation_key``
        is invalid.

        """
        new_email = "john@newexample.com"
        user = User.objects.get(pk=1)
        user.userena_signup.change_email(new_email)

        # Verify email with wrong nonce
        self.assertFalse(UserenaSignup.objects.confirm_email("sha1"))

        # Correct nonce, but non-existent in db.
        self.assertFalse(UserenaSignup.objects.confirm_email(10 * "a1b2"))

    def test_delete_expired_users(self):
        """
        Test if expired users are deleted from the database.

        """
        expired_user = UserenaSignup.objects.create_user(**self.user_info)
        expired_user.date_joined -= datetime.timedelta(
            days=userena_settings.USERENA_ACTIVATION_DAYS + 1
        )
        expired_user.save()

        deleted_users = UserenaSignup.objects.delete_expired_users()

        self.assertEqual(deleted_users[0].username, "alice")


class UserenaManagersIssuesTests(TestCase):
    fixtures = ["users"]

    def test_issue_455_printing_user_model_from_userena_signup_objects_create_user(
        self
    ):
        """
        Issue: https://github.com/bread-and-pepper/django-userena/issues/455
        """
        user = UserenaSignup.objects.create_user(
            "test", "test@t.com", "test", active=True, send_email=False
        )
        # printing of user should not raise any exception
        print(user)



import pytest
from django.core.management import call_command


@pytest.mark.django_db
def test_make_migration(capsys):
    """ Ensure that migrations do not need to be made """
    call_command("makemigrations")
    out, err = capsys.readouterr()
    assert out == "No changes detected\n"
    assert err == ""



from django.contrib.auth import get_user_model
from django.test import TestCase
from django.utils.translation import gettext_lazy as _, override

from userena import forms
from userena import settings as userena_settings


class SignupFormTests(TestCase):
    """ Test the signup form. """

    fixtures = ["users"]

    def test_signup_form(self):
        """
        Test that the ``SignupForm`` checks for unique usernames and unique
        e-mail addresses.

        """
        invalid_data_dicts = [
            # Non-alphanumeric username.
            {
                "data": {
                    "username": "foo@bar",
                    "email": "foo@example.com",
                    "password": "foo",
                    "password2": "foo",
                    "tos": "on",
                },
                "error": (
                    "username",
                    [
                        _(
                            "Username must contain only letters, numbers, dots and underscores."
                        )
                    ],
                ),
            },
            # Password is not the same
            {
                "data": {
                    "username": "katy-",
                    "email": "katy@newexample.com",
                    "password1": "foo",
                    "password2": "foo2",
                    "tos": "on",
                },
                "error": ("__all__", [_("The two password fields didn't match.")]),
            },
            # Already taken username
            {
                "data": {
                    "username": "john",
                    "email": "john@newexample.com",
                    "password1": "foo",
                    "password2": "foo",
                    "tos": "on",
                },
                "error": ("username", [_("This username is already taken.")]),
            },
            # Forbidden username
            {
                "data": {
                    "username": "SignUp",
                    "email": "foo@example.com",
                    "password": "foo",
                    "password2": "foo2",
                    "tos": "on",
                },
                "error": ("username", [_("This username is not allowed.")]),
            },
            # Already taken email
            {
                "data": {
                    "username": "alice",
                    "email": "john@example.com",
                    "password": "foo",
                    "password2": "foo",
                    "tos": "on",
                },
                "error": (
                    "email",
                    [
                        _(
                            "This email is already in use. Please supply a different email."
                        )
                    ],
                ),
            },
        ]

        # Override locale settings since we are checking for existence of error
        # messaged written in english. Note: it should not be necessasy but
        # we have experienced such locale issues during tests on Travis builds.
        # See: https://github.com/bread-and-pepper/django-userena/issues/446
        with override("en"):
            for invalid_dict in invalid_data_dicts:
                form = forms.SignupForm(data=invalid_dict["data"])
                self.assertFalse(form.is_valid())
                self.assertEqual(
                    form.errors[invalid_dict["error"][0]], invalid_dict["error"][1]
                )

        # And finally, a valid form.
        form = forms.SignupForm(
            data={
                "username": "foo.bla",
                "email": "foo@example.com",
                "password1": "foo",
                "password2": "foo",
                "tos": "on",
            }
        )

        self.assertTrue(form.is_valid())


class AuthenticationFormTests(TestCase):
    """ Test the ``AuthenticationForm`` """

    fixtures = ["users"]

    def test_signin_form(self):
        """
        Check that the ``SigninForm`` requires both identification and password

        """
        invalid_data_dicts = [
            {
                "data": {"identification": "", "password": "inhalefish"},
                "error": (
                    "identification",
                    ["Either supply us with your email or username."],
                ),
            },
            {
                "data": {"identification": "john", "password": "inhalefish"},
                "error": (
                    "__all__",
                    [
                        "Please enter a correct username or email and password. Note that both fields are case-sensitive."
                    ],
                ),
            },
        ]

        # Override locale settings since we are checking for existence of error
        # messaged written in english. Note: it should not be necessasy but
        # we have experienced such locale issues during tests on Travis builds.
        # See: https://github.com/bread-and-pepper/django-userena/issues/446
        with override("en"):
            for invalid_dict in invalid_data_dicts:
                form = forms.AuthenticationForm(data=invalid_dict["data"])
                self.assertFalse(form.is_valid())
                self.assertEqual(
                    form.errors[invalid_dict["error"][0]], invalid_dict["error"][1]
                )

        valid_data_dicts = [
            {"identification": "john", "password": "blowfish"},
            {"identification": "john@example.com", "password": "blowfish"},
        ]

        for valid_dict in valid_data_dicts:
            form = forms.AuthenticationForm(valid_dict)
            self.assertTrue(form.is_valid())

    def test_signin_form_email(self):
        """
        Test that the signin form has a different label is
        ``USERENA_WITHOUT_USERNAME`` is set to ``True``

        """
        userena_settings.USERENA_WITHOUT_USERNAMES = True

        form = forms.AuthenticationForm(
            data={"identification": "john", "password": "blowfish"}
        )

        correct_label = "Email"
        self.assertEqual(form.fields["identification"].label, correct_label)

        # Restore default settings
        userena_settings.USERENA_WITHOUT_USERNAMES = False


class SignupFormOnlyEmailTests(TestCase):
    """
    Test the :class:`SignupFormOnlyEmail`.

    This is the same form as :class:`SignupForm` but doesn't require an
    username for a successfull signup.

    """

    fixtures = ["users"]

    def test_signup_form_only_email(self):
        """
        Test that the form has no username field. And that the username is
        generated in the save method

        """
        valid_data = {
            "email": "hans@gretel.com",
            "password1": "blowfish",
            "password2": "blowfish",
        }

        form = forms.SignupFormOnlyEmail(data=valid_data)

        # Should have no username field
        self.assertFalse(form.fields.get("username", False))

        # Form should be valid.
        self.assertTrue(form.is_valid())

        # Creates an unique username
        user = form.save()

        self.assertTrue(len(user.username), 5)


class ChangeEmailFormTests(TestCase):
    """ Test the ``ChangeEmailForm`` """

    fixtures = ["users"]

    def test_change_email_form(self):
        user = get_user_model().objects.get(pk=1)
        invalid_data_dicts = [
            # No change in e-mail address
            {
                "data": {"email": "john@example.com"},
                "error": ("email", ["You're already known under this email."]),
            },
            # An e-mail address used by another
            {
                "data": {"email": "jane@example.com"},
                "error": (
                    "email",
                    ["This email is already in use. Please supply a different email."],
                ),
            },
        ]

        # Override locale settings since we are checking for existence of error
        # messaged written in english. Note: it should not be necessasy but
        # we have experienced such locale issues during tests on Travis builds.
        # See: https://github.com/bread-and-pepper/django-userena/issues/446
        with override("en"):
            for invalid_dict in invalid_data_dicts:
                form = forms.ChangeEmailForm(user, data=invalid_dict["data"])
                self.assertFalse(form.is_valid())
                self.assertEqual(
                    form.errors[invalid_dict["error"][0]], invalid_dict["error"][1]
                )

        # Test a valid post
        form = forms.ChangeEmailForm(user, data={"email": "john@newexample.com"})
        self.assertTrue(form.is_valid())

    def test_form_init(self):
        """ The form must be initialized with a ``User`` instance. """
        self.assertRaises(TypeError, forms.ChangeEmailForm, None)


class EditAccountFormTest(TestCase):
    """ Test the ``EditAccountForm`` """

    pass



from django.contrib.auth import views as auth_views
from django.urls import re_path, reverse_lazy

from userena import settings as userena_settings
from userena import views as userena_views

urlpatterns = [
    # Signup, signin and signout
    re_path(r"^signup/$", userena_views.signup, name="userena_signup"),
    re_path(r"^signin/$", userena_views.signin, name="userena_signin"),
    re_path(r"^signout/$", userena_views.SignoutView.as_view(), name="userena_signout"),
    # Reset password
    re_path(
        r"^password/reset/$",
        auth_views.PasswordResetView.as_view(
            template_name="userena/password_reset_form.html",
            email_template_name="userena/emails/password_reset_message.txt",
            extra_context={
                "without_usernames": userena_settings.USERENA_WITHOUT_USERNAMES
            },
            success_url=reverse_lazy("userena_password_reset_done"),
        ),
        name="userena_password_reset",
    ),
    re_path(
        r"^password/reset/done/$",
        auth_views.PasswordResetDoneView.as_view(
            template_name="userena/password_reset_done.html"
        ),
        name="userena_password_reset_done",
    ),
    re_path(
        r"^password/reset/confirm/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>.+)/$",
        auth_views.PasswordResetConfirmView.as_view(
            template_name="userena/password_reset_confirm_form.html",
            success_url=reverse_lazy("userena_password_reset_complete"),
        ),
        name="userena_password_reset_confirm",
    ),
    re_path(
        r"^password/reset/confirm/complete/$",
        auth_views.PasswordResetCompleteView.as_view(
            template_name="userena/password_reset_complete.html"
        ),
        name="userena_password_reset_complete",
    ),
    # Signup
    re_path(
        r"^(?P<username>[\@\.\+\w-]+)/signup/complete/$",
        userena_views.direct_to_user_template,
        {
            "template_name": "userena/signup_complete.html",
            "extra_context": {
                "userena_activation_required": userena_settings.USERENA_ACTIVATION_REQUIRED,
                "userena_activation_days": userena_settings.USERENA_ACTIVATION_DAYS,
            },
        },
        name="userena_signup_complete",
    ),
    # Activate
    re_path(
        r"^activate/(?P<activation_key>\w+)/$",
        userena_views.activate,
        name="userena_activate",
    ),
    # Retry activation
    re_path(
        r"^activate/retry/(?P<activation_key>\w+)/$",
        userena_views.activate_retry,
        name="userena_activate_retry",
    ),
    # Activate pending
    re_path(
        r"^(?P<username>[\@\.\+\w-]+)/pending/$",
        userena_views.activate_pending,
        name="userena_activate_pending",
    ),
    # Change email and confirm it
    re_path(
        r"^(?P<username>[\@\.\+\w-]+)/email/$",
        userena_views.email_change,
        name="userena_email_change",
    ),
    re_path(
        r"^(?P<username>[\@\.\+\w-]+)/email/complete/$",
        userena_views.direct_to_user_template,
        {"template_name": "userena/email_change_complete.html"},
        name="userena_email_change_complete",
    ),
    re_path(
        r"^(?P<username>[\@\.\+\w-]+)/confirm-email/complete/$",
        userena_views.direct_to_user_template,
        {"template_name": "userena/email_confirm_complete.html"},
        name="userena_email_confirm_complete",
    ),
    re_path(
        r"^confirm-email/(?P<confirmation_key>\w+)/$",
        userena_views.email_confirm,
        name="userena_email_confirm",
    ),
    # Disabled account
    re_path(
        r"^(?P<username>[\@\.\+\w-]+)/disabled/$",
        userena_views.disabled_account,
        name="userena_disabled",
    ),
    # Change password
    re_path(
        r"^(?P<username>[\@\.\+\w-]+)/password/$",
        userena_views.password_change,
        name="userena_password_change",
    ),
    re_path(
        r"^(?P<username>[\@\.\+\w-]+)/password/complete/$",
        userena_views.direct_to_user_template,
        {"template_name": "userena/password_complete.html"},
        name="userena_password_change_complete",
    ),
    # Edit profile
    re_path(
        r"^(?P<username>[\@\.\+\w-]+)/edit/$",
        userena_views.profile_edit,
        name="userena_profile_edit",
    ),
    # View profiles
    re_path(
        r"^(?P<username>(?!(signout|signup|signin)/)[\@\.\+\w-]+)/$",
        userena_views.profile_detail,
        name="userena_profile_detail",
    ),
    re_path(
        r"^page/(?P<page>[0-9]+)/$",
        userena_views.ProfileListView.as_view(),
        name="userena_profile_list_paginated",
    ),
    re_path(
        r"^$", userena_views.ProfileListView.as_view(), name="userena_profile_list"
    ),
]



import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):
    def forwards(self, orm):

        # Adding model 'MessageContact'
        db.create_table(
            "umessages_messagecontact",
            (
                ("id", self.gf("django.db.models.fields.AutoField")(primary_key=True)),
                (
                    "um_from_user",
                    self.gf("django.db.models.fields.related.ForeignKey")(
                        related_name="um_from_users", to=orm["auth.User"]
                    ),
                ),
                (
                    "um_to_user",
                    self.gf("django.db.models.fields.related.ForeignKey")(
                        related_name="um_to_users", to=orm["auth.User"]
                    ),
                ),
                (
                    "latest_message",
                    self.gf("django.db.models.fields.related.ForeignKey")(
                        to=orm["umessages.Message"]
                    ),
                ),
            ),
        )
        db.send_create_signal("umessages", ["MessageContact"])

        # Adding unique constraint on 'MessageContact', fields ['um_from_user', 'um_to_user']
        db.create_unique(
            "umessages_messagecontact", ["um_from_user_id", "um_to_user_id"]
        )

        # Adding model 'MessageRecipient'
        db.create_table(
            "umessages_messagerecipient",
            (
                ("id", self.gf("django.db.models.fields.AutoField")(primary_key=True)),
                (
                    "user",
                    self.gf("django.db.models.fields.related.ForeignKey")(
                        to=orm["auth.User"]
                    ),
                ),
                (
                    "message",
                    self.gf("django.db.models.fields.related.ForeignKey")(
                        to=orm["umessages.Message"]
                    ),
                ),
                (
                    "read_at",
                    self.gf("django.db.models.fields.DateTimeField")(
                        null=True, blank=True
                    ),
                ),
                (
                    "deleted_at",
                    self.gf("django.db.models.fields.DateTimeField")(
                        null=True, blank=True
                    ),
                ),
            ),
        )
        db.send_create_signal("umessages", ["MessageRecipient"])

        # Adding model 'Message'
        db.create_table(
            "umessages_message",
            (
                ("id", self.gf("django.db.models.fields.AutoField")(primary_key=True)),
                ("body", self.gf("django.db.models.fields.TextField")()),
                (
                    "sender",
                    self.gf("django.db.models.fields.related.ForeignKey")(
                        related_name="sent_messages", to=orm["auth.User"]
                    ),
                ),
                (
                    "sent_at",
                    self.gf("django.db.models.fields.DateTimeField")(
                        auto_now_add=True, blank=True
                    ),
                ),
                (
                    "sender_deleted_at",
                    self.gf("django.db.models.fields.DateTimeField")(
                        null=True, blank=True
                    ),
                ),
            ),
        )
        db.send_create_signal("umessages", ["Message"])

    def backwards(self, orm):

        # Removing unique constraint on 'MessageContact', fields ['um_from_user', 'um_to_user']
        db.delete_unique(
            "umessages_messagecontact", ["um_from_user_id", "um_to_user_id"]
        )

        # Deleting model 'MessageContact'
        db.delete_table("umessages_messagecontact")

        # Deleting model 'MessageRecipient'
        db.delete_table("umessages_messagerecipient")

        # Deleting model 'Message'
        db.delete_table("umessages_message")

    models = {
        "auth.group": {
            "Meta": {"object_name": "Group"},
            "id": ("django.db.models.fields.AutoField", [], {"primary_key": "True"}),
            "name": (
                "django.db.models.fields.CharField",
                [],
                {"unique": "True", "max_length": "80"},
            ),
            "permissions": (
                "django.db.models.fields.related.ManyToManyField",
                [],
                {
                    "to": "orm['auth.Permission']",
                    "symmetrical": "False",
                    "blank": "True",
                },
            ),
        },
        "auth.permission": {
            "Meta": {
                "ordering": "('content_type__app_label', 'content_type__model', 'codename')",
                "unique_together": "(('content_type', 'codename'),)",
                "object_name": "Permission",
            },
            "codename": (
                "django.db.models.fields.CharField",
                [],
                {"max_length": "100"},
            ),
            "content_type": (
                "django.db.models.fields.related.ForeignKey",
                [],
                {"to": "orm['contenttypes.ContentType']"},
            ),
            "id": ("django.db.models.fields.AutoField", [], {"primary_key": "True"}),
            "name": ("django.db.models.fields.CharField", [], {"max_length": "50"}),
        },
        "auth.user": {
            "Meta": {"object_name": "User"},
            "date_joined": (
                "django.db.models.fields.DateTimeField",
                [],
                {"default": "datetime.datetime.now"},
            ),
            "email": (
                "django.db.models.fields.EmailField",
                [],
                {"max_length": "75", "blank": "True"},
            ),
            "first_name": (
                "django.db.models.fields.CharField",
                [],
                {"max_length": "30", "blank": "True"},
            ),
            "groups": (
                "django.db.models.fields.related.ManyToManyField",
                [],
                {"to": "orm['auth.Group']", "symmetrical": "False", "blank": "True"},
            ),
            "id": ("django.db.models.fields.AutoField", [], {"primary_key": "True"}),
            "is_active": (
                "django.db.models.fields.BooleanField",
                [],
                {"default": "True"},
            ),
            "is_staff": (
                "django.db.models.fields.BooleanField",
                [],
                {"default": "False"},
            ),
            "is_superuser": (
                "django.db.models.fields.BooleanField",
                [],
                {"default": "False"},
            ),
            "last_login": (
                "django.db.models.fields.DateTimeField",
                [],
                {"default": "datetime.datetime.now"},
            ),
            "last_name": (
                "django.db.models.fields.CharField",
                [],
                {"max_length": "30", "blank": "True"},
            ),
            "password": (
                "django.db.models.fields.CharField",
                [],
                {"max_length": "128"},
            ),
            "user_permissions": (
                "django.db.models.fields.related.ManyToManyField",
                [],
                {
                    "to": "orm['auth.Permission']",
                    "symmetrical": "False",
                    "blank": "True",
                },
            ),
            "username": (
                "django.db.models.fields.CharField",
                [],
                {"unique": "True", "max_length": "30"},
            ),
        },
        "contenttypes.contenttype": {
            "Meta": {
                "ordering": "('name',)",
                "unique_together": "(('app_label', 'model'),)",
                "object_name": "ContentType",
                "db_table": "'django_content_type'",
            },
            "app_label": (
                "django.db.models.fields.CharField",
                [],
                {"max_length": "100"},
            ),
            "id": ("django.db.models.fields.AutoField", [], {"primary_key": "True"}),
            "model": ("django.db.models.fields.CharField", [], {"max_length": "100"}),
            "name": ("django.db.models.fields.CharField", [], {"max_length": "100"}),
        },
        "umessages.message": {
            "Meta": {"ordering": "['-sent_at']", "object_name": "Message"},
            "body": ("django.db.models.fields.TextField", [], {}),
            "id": ("django.db.models.fields.AutoField", [], {"primary_key": "True"}),
            "recipients": (
                "django.db.models.fields.related.ManyToManyField",
                [],
                {
                    "related_name": "'received_messages'",
                    "symmetrical": "False",
                    "through": "orm['umessages.MessageRecipient']",
                    "to": "orm['auth.User']",
                },
            ),
            "sender": (
                "django.db.models.fields.related.ForeignKey",
                [],
                {"related_name": "'sent_messages'", "to": "orm['auth.User']"},
            ),
            "sender_deleted_at": (
                "django.db.models.fields.DateTimeField",
                [],
                {"null": "True", "blank": "True"},
            ),
            "sent_at": (
                "django.db.models.fields.DateTimeField",
                [],
                {"auto_now_add": "True", "blank": "True"},
            ),
        },
        "umessages.messagecontact": {
            "Meta": {
                "ordering": "['latest_message']",
                "unique_together": "(('um_from_user', 'um_to_user'),)",
                "object_name": "MessageContact",
            },
            "um_from_user": (
                "django.db.models.fields.related.ForeignKey",
                [],
                {"related_name": "'um_from_users'", "to": "orm['auth.User']"},
            ),
            "id": ("django.db.models.fields.AutoField", [], {"primary_key": "True"}),
            "latest_message": (
                "django.db.models.fields.related.ForeignKey",
                [],
                {"to": "orm['umessages.Message']"},
            ),
            "um_to_user": (
                "django.db.models.fields.related.ForeignKey",
                [],
                {"related_name": "'um_to_users'", "to": "orm['auth.User']"},
            ),
        },
        "umessages.messagerecipient": {
            "Meta": {"object_name": "MessageRecipient"},
            "deleted_at": (
                "django.db.models.fields.DateTimeField",
                [],
                {"null": "True", "blank": "True"},
            ),
            "id": ("django.db.models.fields.AutoField", [], {"primary_key": "True"}),
            "message": (
                "django.db.models.fields.related.ForeignKey",
                [],
                {"to": "orm['umessages.Message']"},
            ),
            "read_at": (
                "django.db.models.fields.DateTimeField",
                [],
                {"null": "True", "blank": "True"},
            ),
            "user": (
                "django.db.models.fields.related.ForeignKey",
                [],
                {"to": "orm['auth.User']"},
            ),
        },
    }

    complete_apps = ["umessages"]



from functools import WRAPPER_ASSIGNMENTS, wraps

from django.http import HttpResponsePermanentRedirect
from django.conf import settings

from userena import settings as userena_settings


def secure_required(view_func):
    """
    Decorator to switch an url from http to https.

    If a view is accessed through http and this decorator is applied to that
    view, than it will return a permanent redirect to the secure (https)
    version of the same view.

    The decorator also must check that ``USERENA_USE_HTTPS`` is enabled. If
    disabled, it should not redirect to https because the project doesn't
    support it.

    """

    def _wrapped_view(request, *args, **kwargs):
        if not request.is_secure():
            if getattr(
                settings,
                "USERENA_USE_HTTPS",
                userena_settings.DEFAULT_USERENA_USE_HTTPS,
            ):
                request_url = request.build_absolute_uri(request.get_full_path())
                secure_url = request_url.replace("http://", "https://")
                return HttpResponsePermanentRedirect(secure_url)
        return view_func(request, *args, **kwargs)

    return wraps(view_func, assigned=WRAPPER_ASSIGNMENTS)(_wrapped_view)



from django.db import models
from django.utils.translation import gettext_lazy as _

from userena.utils import truncate_words
from userena.contrib.umessages.managers import (
    MessageManager,
    MessageContactManager,
    MessageRecipientManager,
)
from userena.utils import user_model_label


class MessageContact(models.Model):
    """
    Contact model.

    A contact is a user to whom a user has send a message to or
    received a message from.

    """

    um_from_user = models.ForeignKey(
        user_model_label,
        verbose_name=_("from user"),
        related_name=("um_from_users"),
        on_delete=models.CASCADE,
    )

    um_to_user = models.ForeignKey(
        user_model_label,
        verbose_name=_("to user"),
        related_name=("um_to_users"),
        on_delete=models.CASCADE,
    )

    latest_message = models.ForeignKey(
        "Message", verbose_name=_("latest message"), on_delete=models.CASCADE
    )

    objects = MessageContactManager()

    class Meta:
        unique_together = ("um_from_user", "um_to_user")
        ordering = ["latest_message"]
        verbose_name = _("contact")
        verbose_name_plural = _("contacts")

    def __str__(self):
        return _("%(um_from_user)s and %(um_to_user)s") % {
            "um_from_user": self.um_from_user.username,
            "um_to_user": self.um_to_user.username,
        }

    def opposite_user(self, user):
        """
        Returns the user opposite of the user that is given

        :param user:
            A Django :class:`User`.

        :return:
            A Django :class:`User`.

        """
        if self.um_from_user == user:
            return self.um_to_user
        else:
            return self.um_from_user


class MessageRecipient(models.Model):
    """
    Intermediate model to allow per recipient marking as
    deleted, read etc. of a message.

    """

    user = models.ForeignKey(
        user_model_label, verbose_name=_("recipient"), on_delete=models.CASCADE
    )

    message = models.ForeignKey(
        "Message", verbose_name=_("message"), on_delete=models.CASCADE
    )

    read_at = models.DateTimeField(_("read at"), null=True, blank=True)

    deleted_at = models.DateTimeField(_("recipient deleted at"), null=True, blank=True)

    objects = MessageRecipientManager()

    class Meta:
        verbose_name = _("recipient")
        verbose_name_plural = _("recipients")

    def __str__(self):
        return _("%(message)s") % {"message": self.message}

    def is_read(self):
        """ Returns a boolean whether the recipient has read the message """
        return self.read_at is None


class Message(models.Model):
    """ Private message model, from user to user(s) """

    body = models.TextField(_("body"))

    sender = models.ForeignKey(
        user_model_label,
        related_name="sent_messages",
        verbose_name=_("sender"),
        on_delete=models.CASCADE,
    )

    recipients = models.ManyToManyField(
        user_model_label,
        through="MessageRecipient",
        related_name="received_messages",
        verbose_name=_("recipients"),
    )

    sent_at = models.DateTimeField(_("sent at"), auto_now_add=True)

    sender_deleted_at = models.DateTimeField(
        _("sender deleted at"), null=True, blank=True
    )

    objects = MessageManager()

    class Meta:
        ordering = ["-sent_at"]
        verbose_name = _("message")
        verbose_name_plural = _("messages")

    def __str__(self):
        """ Human representation, displaying first ten words of the body. """
        truncated_body = truncate_words(self.body, 10)
        return "%(truncated_body)s" % {"truncated_body": truncated_body}

    def save_recipients(self, um_to_user_list):
        """
        Save the recipients for this message

        :param um_to_user_list:
            A list which elements are :class:`User` to whom the message is for.

        :return:
            Boolean indicating if any users are saved.

        """
        created = False
        for user in um_to_user_list:
            MessageRecipient.objects.create(user=user, message=self)
            created = True
        return created

    def update_contacts(self, um_to_user_list):
        """
        Updates the contacts that are used for this message.

        :param um_to_user_list:
            List of Django :class:`User`.

        :return:
            A boolean if a user is contact is updated.

        """
        updated = False
        for user in um_to_user_list:
            MessageContact.objects.update_contact(self.sender, user, self)
            updated = True
        return updated



from django.contrib.auth import get_user_model
from django.test import TestCase
from django.urls import reverse

from userena.contrib.umessages.forms import ComposeForm
from userena.contrib.umessages.models import Message, MessageRecipient

User = get_user_model()


class MessagesViewsTests(TestCase):
    fixtures = ["users", "messages"]

    def _test_login(self, named_url, **kwargs):
        """ Test that the view requires login """
        response = self.client.get(reverse(named_url, **kwargs))
        self.assertEqual(response.status_code, 302)

    def test_compose(self):
        """ A ``GET`` to the compose view """
        # Login is required.
        self._test_login("userena_umessages_compose")

        # Sign in
        client = self.client.login(username="john", password="blowfish")
        response = self.client.get(reverse("userena_umessages_compose"))

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "umessages/message_form.html")

        self.assertTrue(isinstance(response.context["form"], ComposeForm))

    def test_compose_post(self):
        """ ``POST`` to the compose view """
        client = self.client.login(username="john", password="blowfish")

        valid_data = {"to": "john", "body": "Hi"}

        # Check for a normal redirect
        response = self.client.post(
            reverse("userena_umessages_compose"), data=valid_data
        )

        self.assertRedirects(
            response, reverse("userena_umessages_detail", kwargs={"username": "john"})
        )

        # Check for a requested redirect
        valid_data["next"] = reverse("userena_umessages_compose")
        response = self.client.post(
            reverse("userena_umessages_compose"), data=valid_data
        )
        self.assertRedirects(response, valid_data["next"])

    def test_compose_recipients(self):
        """ A ``GET`` to the compose view with recipients """
        client = self.client.login(username="john", password="blowfish")

        valid_recipients = "john+jane"
        invalid_recipients = "johny+jane"

        # Test valid recipients
        response = self.client.get(
            reverse(
                "userena_umessages_compose_to", kwargs={"recipients": valid_recipients}
            )
        )

        self.assertEqual(response.status_code, 200)

        # Test the users
        jane = User.objects.get(username="jane")
        john = User.objects.get(username="john")
        self.assertEqual(response.context["recipients"][0], jane)
        self.assertEqual(response.context["recipients"][1], john)

        # Test that the initial data of the form is set.
        self.assertEqual(response.context["form"].initial["to"], [jane, john])

    def test_message_detail(self):
        """ A ``GET`` to the detail view """
        self._test_login("userena_umessages_detail", kwargs={"message_id": 2})

        # Sign in
        client = self.client.login(username="jane", password="blowfish")
        response = self.client.get(
            reverse("userena_umessages_detail", kwargs={"message_id": 1})
        )

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "umessages/message_detail.html")

        # Test that the message is read.
        jane = User.objects.get(pk=2)
        mr = MessageRecipient.objects.get(message=Message.objects.get(pk=1), user=jane)
        self.assertTrue(mr.read_at)

    def test_valid_message_remove(self):
        """ ``POST`` to remove a message """
        # Test that sign in is required
        response = self.client.post(reverse("userena_umessages_remove"))
        self.assertEqual(response.status_code, 302)

        # Sign in
        client = self.client.login(username="john", password="blowfish")

        # Test that only posts are allowed
        response = self.client.get(reverse("userena_umessages_remove"))
        self.assertEqual(response.status_code, 405)

        # Test a valid post to delete a senders message
        response = self.client.post(
            reverse("userena_umessages_remove"), data={"message_pks": "1"}
        )
        self.assertRedirects(response, reverse("userena_umessages_list"))
        msg = Message.objects.get(pk=1)
        self.assertTrue(msg.sender_deleted_at)

        # Test a valid post to delete a recipients message and a redirect
        client = self.client.login(username="jane", password="blowfish")
        response = self.client.post(
            reverse("userena_umessages_remove"),
            data={"message_pks": "1", "next": reverse("userena_umessages_list")},
        )
        self.assertRedirects(response, reverse("userena_umessages_list"))
        jane = User.objects.get(username="jane")
        mr = msg.messagerecipient_set.get(user=jane, message=msg)
        self.assertTrue(mr.deleted_at)

    def test_invalid_message_remove(self):
        """ ``POST`` to remove an invalid message """
        # Sign in
        client = self.client.login(username="john", password="blowfish")

        bef_len = Message.objects.filter(sender_deleted_at__isnull=False).count()
        response = self.client.post(
            reverse("userena_umessages_remove"), data={"message_pks": ["a", "b"]}
        )

        # The program should play nice, nothing happened.
        af_len = Message.objects.filter(sender_deleted_at__isnull=False).count()
        self.assertRedirects(response, reverse("userena_umessages_list"))
        self.assertEqual(bef_len, af_len)

    def test_valid_message_remove_multiple(self):
        """ ``POST`` to remove multiple messages """
        # Sign in
        client = self.client.login(username="john", password="blowfish")
        response = self.client.post(
            reverse("userena_umessages_remove"), data={"message_pks": [1, 2]}
        )
        self.assertRedirects(response, reverse("userena_umessages_list"))

        # Message #1 and #2 should be deleted
        msg_list = Message.objects.filter(
            pk__in=["1", "2"], sender_deleted_at__isnull=False
        )
        self.assertEqual(msg_list.count(), 2)

    def test_message_unremove(self):
        """ Unremove a message """
        client = self.client.login(username="john", password="blowfish")

        # Delete a message as owner
        response = self.client.post(
            reverse("userena_umessages_unremove"), data={"message_pks": [1]}
        )

        self.assertRedirects(response, reverse("userena_umessages_list"))

        # Delete the message as a recipient
        response = self.client.post(
            reverse("userena_umessages_unremove"), data={"message_pks": [2]}
        )

        self.assertRedirects(response, reverse("userena_umessages_list"))

    def test_message_list(self):
        """ ``GET`` the message list for a user """
        self._test_login("userena_umessages_list")

        client = self.client.login(username="john", password="blowfish")
        response = self.client.get(reverse("userena_umessages_list"))
        self.assertEqual(response.status_code, 200)

        self.assertTemplateUsed(response, "umessages/message_list.html")

    def test_message_detail(self):
        """ ``GET`` to a detail page between two users """
        self._test_login("userena_umessages_detail", kwargs={"username": "jane"})
        client = self.client.login(username="john", password="blowfish")

        response = self.client.get(
            reverse("userena_umessages_detail", kwargs={"username": "jane"})
        )

        self.assertEqual(response.status_code, 200)

        self.assertTemplateUsed(response, "umessages/message_detail.html")

        # Check that all the messages are marked as read.
        john = User.objects.get(pk=1)
        jane = User.objects.get(pk=2)
        unread_messages = MessageRecipient.objects.filter(
            user=john, message__sender=jane, read_at__isnull=True
        )

        self.assertEqual(len(unread_messages), 0)



from django.contrib.auth import get_user_model
from django.test import TestCase

from userena.contrib.umessages.models import Message, MessageContact, MessageRecipient

User = get_user_model()


class MessageManagerTests(TestCase):
    fixtures = ["users", "messages"]

    def test_get_conversation(self):
        """ Test that the conversation is returned between two users """
        user_1 = User.objects.get(pk=1)
        user_2 = User.objects.get(pk=2)

        messages = Message.objects.get_conversation_between(user_1, user_2)


class MessageRecipientManagerTest(TestCase):
    fixtures = ["users", "messages"]

    def test_count_unread_messages_for(self):
        """ Test the unread messages count for user """
        jane = User.objects.get(pk=2)

        # Jane has one unread message from john
        unread_messages = MessageRecipient.objects.count_unread_messages_for(jane)

        self.assertEqual(unread_messages, 1)

    def test_count_unread_messages_between(self):
        """ Test the unread messages count between two users """
        john = User.objects.get(pk=1)
        jane = User.objects.get(pk=2)

        # Jane should have one unread message from john
        unread_messages = MessageRecipient.objects.count_unread_messages_between(
            jane, john
        )

        self.assertEqual(unread_messages, 1)


class MessageContactManagerTest(TestCase):
    fixtures = ["users", "messages"]

    def test_get_contacts_for(self):
        """ Test if the correct contacts are returned """
        john = User.objects.get(pk=1)
        contacts = MessageContact.objects.get_contacts_for(john)

        # There is only one contact for John, and that's Jane.
        self.assertEqual(len(contacts), 1)

        jane = User.objects.get(pk=2)
        self.assertEqual(contacts[0].um_to_user, jane)



import re, six
from urllib.parse import urlparse, parse_qs

from django.test import TestCase
from django.conf import settings
from django.contrib.auth import get_user_model

from userena.utils import (
    get_gravatar,
    signin_redirect,
    get_profile_model,
    get_protocol,
    generate_nonce,
)
from userena.compat import SiteProfileNotAvailable


class UtilsTests(TestCase):
    """ Test the extra utils methods """

    fixtures = ["users"]

    def test_generate_nonce(self):
        h1 = generate_nonce()
        h2 = generate_nonce()
        h3 = generate_nonce()
        # Check valid activation key
        self.assertTrue(re.match(r"^[\w]{40}$", h1))
        self.assertTrue(re.match(r"^[\w]{40}$", h2))
        self.assertTrue(re.match(r"^[\w]{40}$", h3))
        self.assertNotEqual(h1, h2)
        self.assertNotEqual(h2, h3)
        self.assertNotEqual(h1, h3)

    def test_get_gravatar(self):
        template = "s=%(size)s&d=%(type)s"

        # Check the defaults.
        parsed = urlparse(get_gravatar("alice@example.com"))
        self.assertEqual(
            parse_qs(parsed.query),
            parse_qs(template % {"size": 80, "type": "identicon"}),
        )

        # Check different size
        parsed = urlparse(get_gravatar("alice@example.com", size=200))
        self.assertEqual(
            parse_qs(parsed.query),
            parse_qs(template % {"size": 200, "type": "identicon"}),
        )

        # Check different default
        parsed = urlparse(get_gravatar("alice@example.com", default="404"))
        self.assertEqual(
            parse_qs(parsed.query), parse_qs(template % {"size": 80, "type": "404"})
        )

    def test_signin_redirect(self):
        """
        Test redirect function which should redirect the user after a
        succesfull signin.

        """
        # Test with a requested redirect
        self.assertEqual(signin_redirect(redirect="/accounts/"), "/accounts/")

        # Test with only the user specified
        user = get_user_model().objects.get(pk=1)
        self.assertEqual(signin_redirect(user=user), "/accounts/%s/" % user.username)

        # The ultimate fallback, probably never used
        self.assertEqual(signin_redirect(), settings.LOGIN_REDIRECT_URL)

    def test_get_profile_model(self):
        """
        Test if the correct profile model is returned when
        ``get_profile_model()`` is called.

        """
        # A non existent model should also raise ``SiteProfileNotAvailable``
        # error.
        with self.settings(AUTH_PROFILE_MODULE="userena.FakeProfile"):
            self.assertRaises(SiteProfileNotAvailable, get_profile_model)

        # An error should be raised when there is no ``AUTH_PROFILE_MODULE``
        # supplied.
        with self.settings(AUTH_PROFILE_MODULE=None):
            self.assertRaises(SiteProfileNotAvailable, get_profile_model)

    def test_get_protocol(self):
        """ Test if the correct protocol is returned """
        self.assertEqual(get_protocol(), "http")

        with self.settings(USERENA_USE_HTTPS=True):
            self.assertEqual(get_protocol(), "https")



import re

from django.conf import settings
from django.template.loader import render_to_string
from django.utils.translation import gettext as _
from django.core.mail import EmailMultiAlternatives

from userena import settings as userena_settings

from html2text import html2text


def send_mail(
    subject,
    message_plain,
    message_html,
    email_from,
    email_to,
    custom_headers={},
    attachments=(),
):
    """
    Build the email as a multipart message containing
    a multipart alternative for text (plain, HTML) plus
    all the attached files.
    """
    if not message_plain and not message_html:
        raise ValueError(_("Either message_plain or message_html should be not None"))

    if not message_plain:
        message_plain = html2text(message_html)

    message = {}

    message["subject"] = subject
    message["body"] = message_plain
    message["from_email"] = email_from
    message["to"] = email_to
    if attachments:
        message["attachments"] = attachments
    if custom_headers:
        message["headers"] = custom_headers

    msg = EmailMultiAlternatives(**message)
    if message_html:
        msg.attach_alternative(message_html, "text/html")
    msg.send()


def wrap_attachment():
    pass


class UserenaConfirmationMail(object):

    _message_txt = "userena/emails/{0}_email_message{1}.txt"
    _message_html = "userena/emails/{0}_email_message{1}.html"
    _subject_txt = "userena/emails/{0}_email_subject{1}.txt"

    def __init__(self, context):
        self.context = context

    def generate_mail(self, type_mail, version=""):
        self.type_mail = type_mail
        self.message_txt = self._message_txt.format(type_mail, version)
        self.message_html = self._message_html.format(type_mail, version)
        self.subject_txt = self._subject_txt.format(type_mail, version)
        self.subject = self._subject()
        self.message_html = self._message_in_html()
        self.message = self._message_in_txt()

    def send_mail(self, email):
        send_mail(
            self.subject,
            self.message,
            self.message_html,
            settings.DEFAULT_FROM_EMAIL,
            [email],
        )

    def _message_in_html(self):
        if userena_settings.USERENA_HTML_EMAIL:
            return render_to_string(self.message_html, self.context)
        return None

    def _message_in_txt(self):
        if (
            not userena_settings.USERENA_HTML_EMAIL
            or not self.message_html
            or userena_settings.USERENA_USE_PLAIN_TEMPLATE
        ):
            return render_to_string(self.message_txt, self.context)
        return None

    def _subject(self):
        subject = render_to_string(self.subject_txt, self.context)
        subject = "".join(subject.splitlines())
        return subject



from django import template

from userena.contrib.umessages.models import MessageRecipient

import re

register = template.Library()


class MessageCount(template.Node):
    def __init__(self, um_from_user, var_name, um_to_user=None):
        self.user = template.Variable(um_from_user)
        self.var_name = var_name
        if um_to_user:
            self.um_to_user = template.Variable(um_to_user)
        else:
            self.um_to_user = um_to_user

    def render(self, context):
        try:
            user = self.user.resolve(context)
        except template.VariableDoesNotExist:
            return ""

        if not self.um_to_user:
            message_count = MessageRecipient.objects.count_unread_messages_for(user)

        else:
            try:
                um_to_user = self.um_to_user.resolve(context)
            except template.VariableDoesNotExist:
                return ""

            message_count = MessageRecipient.objects.count_unread_messages_between(
                user, um_to_user
            )

        context[self.var_name] = message_count

        return ""


@register.tag
def get_unread_message_count_for(parser, token):
    """
    Returns the unread message count for a user.

    Syntax::

        {% get_unread_message_count_for [user] as [var_name] %}

    Example usage::

        {% get_unread_message_count_for pero as message_count %}

    """
    try:
        tag_name, arg = token.contents.split(None, 1)
    except ValueError:
        raise template.TemplateSyntaxError(
            "%s tag requires arguments" % token.contents.split()[0]
        )
    m = re.search(r"(.*?) as (\w+)", arg)
    if not m:
        raise template.TemplateSyntaxError("%s tag had invalid arguments" % tag_name)
    user, var_name = m.groups()
    return MessageCount(user, var_name)


@register.tag
def get_unread_message_count_between(parser, token):
    """
    Returns the unread message count between two users.

    Syntax::

        {% get_unread_message_count_between [user] and [user] as [var_name] %}

    Example usage::

        {% get_unread_message_count_between funky and wunki as message_count %}

    """
    try:
        tag_name, arg = token.contents.split(None, 1)
    except ValueError:
        raise template.TemplateSyntaxError(
            "%s tag requires arguments" % token.contents.split()[0]
        )
    m = re.search(r"(.*?) and (.*?) as (\w+)", arg)
    if not m:
        raise template.TemplateSyntaxError("%s tag had invalid arguments" % tag_name)
    um_from_user, um_to_user, var_name = m.groups()
    return MessageCount(um_from_user, var_name, um_to_user)



import re

from django.test import TestCase
from django import forms

from userena.contrib.umessages.fields import CommaSeparatedUserField


class CommaSeparatedTestForm(forms.Form):
    users = CommaSeparatedUserField()

    def __init__(self, *args, **kwargs):
        super(CommaSeparatedTestForm, self).__init__(*args, **kwargs)
        self.fields["users"]._recipient_filter = self.filter_jane

    def filter_jane(self, user):
        if user.username == "jane":
            return False
        return True


class CommaSeperatedFieldTests(TestCase):
    fixtures = ["users"]

    def test_invalid_data(self):
        # Test invalid data supplied to the field.
        invalid_data_dicts = [
            # Empty username
            {"data": {"users": ""}, "error": ("users", ["This field is required."])},
            # No data
            {"data": {}, "error": ("users", ["This field is required."])},
            # A list
            {"data": {"users": []}, "error": ("users", ["This field is required."])},
            # Forbidden username
            {
                "data": {"users": "jane"},
                "error": ("users", ["The following usernames are incorrect: jane."]),
            },
            # Non-existant username
            {
                "data": {"users": "foo"},
                "error": ("users", ["The following usernames are incorrect: foo."]),
            },
            # Multiple invalid usernames
            {
                "data": {"users": "foo, bar"},
                "error": (
                    "users",
                    ["The following usernames are incorrect: (foo|bar), (foo|bar)."],
                ),
            },
            # Valid and invalid
            {
                "data": {"users": "foo, john, bar"},
                "error": (
                    "users",
                    ["The following usernames are incorrect: (foo|bar), (foo|bar)."],
                ),
            },
            # Extra whitespace
            {
                "data": {"users": "foo,    john  "},
                "error": ("users", ["The following usernames are incorrect: foo."]),
            },
        ]
        for invalid_dict in invalid_data_dicts:
            form = CommaSeparatedTestForm(data=invalid_dict["data"])
            self.assertFalse(form.is_valid())
            # self.assertEqual(form.errors[invalid_dict['error'][0]],
            #                  invalid_dict['error'][1])
            self.assertTrue(
                re.match(
                    invalid_dict["error"][1][0],
                    form.errors[invalid_dict["error"][0]][0],
                )
            )



from django.contrib.auth import get_user_model
from django.test import TestCase

from userena.contrib.umessages.models import Message, MessageRecipient, MessageContact
from userena.utils import truncate_words

User = get_user_model()


class MessageContactTests(TestCase):
    fixtures = ["users", "messages"]

    def test_string_formatting(self):
        """ Test the human representation of a message """
        contact = MessageContact.objects.get(pk=1)
        correct_format = "john and jane"
        self.assertEqual(contact.__str__(), correct_format)

    def test_opposite_user(self):
        """ Test if the opposite user is returned """
        contact = MessageContact.objects.get(pk=1)
        john = User.objects.get(pk=1)
        jane = User.objects.get(pk=2)

        # Test the opposites
        self.assertEqual(contact.opposite_user(john), jane)

        self.assertEqual(contact.opposite_user(jane), john)


class MessageModelTests(TestCase):
    fixtures = ["users", "messages"]

    def test_string_formatting(self):
        """ Test the human representation of a message """
        message = Message.objects.get(pk=1)
        truncated_body = truncate_words(message.body, 10)
        self.assertEqual(message.__str__(), truncated_body)


class MessageRecipientModelTest(TestCase):
    fixtures = ["users", "messages"]

    def test_string_formatting(self):
        """ Test the human representation of a recipient """
        recipient = MessageRecipient.objects.get(pk=1)

        valid_unicode = "%s" % (recipient.message)

        self.assertEqual(recipient.__str__(), valid_unicode)

    def test_new(self):
        """ Test if the message that is new is correct """
        new_message = MessageRecipient.objects.get(pk=1)
        read_message = MessageRecipient.objects.get(pk=2)

        self.assertTrue(new_message.is_read())
        self.assertFalse(read_message.is_read())



import datetime
import secrets
from hashlib import md5
from string import ascii_letters, digits

from django.apps import apps
from django.conf import settings
from django.utils.functional import keep_lazy_text
from django.utils.http import urlencode
from django.utils.text import Truncator

from userena import settings as userena_settings
from userena.compat import SiteProfileNotAvailable


def truncate_words(s, num, end_text="..."):
    truncate = end_text and " %s" % end_text or ""
    return Truncator(s).words(num, truncate=truncate)


truncate_words = keep_lazy_text(truncate_words)


def get_gravatar(email, size=80, default="identicon"):
    """ Get's a Gravatar for a email address.

    :param size:
        The size in pixels of one side of the Gravatar's square image.
        Optional, if not supplied will default to ``80``.

    :param default:
        Defines what should be displayed if no image is found for this user.
        Optional argument which defaults to ``identicon``. The argument can be
        a URI to an image or one of the following options:

            ``404``
                Do not load any image if none is associated with the email
                hash, instead return an HTTP 404 (File Not Found) response.

            ``mm``
                Mystery-man, a simple, cartoon-style silhouetted outline of a
                person (does not vary by email hash).

            ``identicon``
                A geometric pattern based on an email hash.

            ``monsterid``
                A generated 'monster' with different colors, faces, etc.

            ``wavatar``
                Generated faces with differing features and backgrounds

    :return: The URI pointing to the Gravatar.

    """
    if userena_settings.USERENA_MUGSHOT_GRAVATAR_SECURE:
        base_url = "https://secure.gravatar.com/avatar/"
    else:
        base_url = "//www.gravatar.com/avatar/"

    gravatar_url = "%(base_url)s%(gravatar_id)s?" % {
        "base_url": base_url,
        "gravatar_id": md5(email.lower().encode("utf-8")).hexdigest(),
    }

    gravatar_url += urlencode({"s": str(size), "d": default})
    return gravatar_url


def signin_redirect(redirect=None, user=None):
    """
    Redirect user after successful sign in.

    First looks for a ``requested_redirect``. If not supplied will fall-back to
    the user specific account page. If all fails, will fall-back to the standard
    Django ``LOGIN_REDIRECT_URL`` setting. Returns a string defining the URI to
    go next.

    :param redirect:
        A value normally supplied by ``next`` form field. Gets preference
        before the default view which requires the user.

    :param user:
        A ``User`` object specifying the user who has just signed in.

    :return: String containing the URI to redirect to.

    """
    if redirect:
        return redirect
    elif user is not None:
        return userena_settings.USERENA_SIGNIN_REDIRECT_URL % {
            "username": user.username
        }
    else:
        return settings.LOGIN_REDIRECT_URL


def generate_nonce():
    """
    Cryptographically generates a 40 char long nonce.

    :return: String containing the nonce.

    """
    alphabet = ascii_letters + digits
    nonce = "".join(secrets.choice(alphabet) for _ in range(40))

    return nonce


def get_profile_model():
    """
    Return the model class for the currently-active user profile
    model, as defined by the ``AUTH_PROFILE_MODULE`` setting.

    :return: The model that is used as profile.

    """
    if (not hasattr(settings, "AUTH_PROFILE_MODULE")) or (
        not settings.AUTH_PROFILE_MODULE
    ):
        raise SiteProfileNotAvailable

    try:
        profile_mod = apps.get_model(*settings.AUTH_PROFILE_MODULE.rsplit(".", 1))
    except LookupError:
        profile_mod = None

    if profile_mod is None:
        raise SiteProfileNotAvailable
    return profile_mod


def get_user_profile(user):
    profile_model = get_profile_model()
    try:
        profile = user.get_profile()
    except AttributeError:
        related_name = profile_model._meta.get_field("user").related_query_name()
        profile = getattr(user, related_name, None)
    except profile_model.DoesNotExist:
        profile = None
    if profile:
        return profile
    return profile_model.objects.create(user=user)


def get_protocol():
    """
    Returns a string with the current protocol.

    This can be either 'http' or 'https' depending on ``USERENA_USE_HTTPS``
    setting.

    """
    protocol = "http"
    if getattr(
        settings, "USERENA_USE_HTTPS", userena_settings.DEFAULT_USERENA_USE_HTTPS
    ):
        protocol = "https"
    return protocol


def get_datetime_now():
    """
    Returns datetime object with current point in time.

    In Django 1.4+ it uses Django's django.utils.timezone.now() which returns
    an aware or naive datetime that represents the current point in time
    when ``USE_TZ`` in project's settings is True or False respectively.
    In older versions of Django it uses datetime.datetime.now().

    """
    try:
        from django.utils import timezone

        return timezone.now()  # pragma: no cover
    except ImportError:  # pragma: no cover
        return datetime.datetime.now()


# Django 1.5 compatibility utilities, providing support for custom User models.
# Since get_user_model() causes a circular import if called when app models are
# being loaded, the user_model_label should be used when possible, with calls
# to get_user_model deferred to execution time

user_model_label = getattr(settings, "AUTH_USER_MODEL", "auth.User")



import django.core.validators
from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend


class UserenaAuthenticationBackend(ModelBackend):
    """
    Custom backend because the user must be able to supply a ``email`` or
    ``username`` to the login form.

    """

    def authenticate(self, request, identification, password=None, check_password=True):
        """
        Authenticates a user through the combination email/username with
        password.

        :param request:
            The authenticate() method of authentication backends requires
            request as the first positional argument from Django 2.1.

        :param identification:
            A string containing the username or e-mail of the user that is
            trying to authenticate.

        :password:
            Optional string containing the password for the user.

        :param check_password:
            Boolean that defines if the password should be checked for this
            user.  Always keep this ``True``. This is only used by userena at
            activation when a user opens a page with a secret hash.

        :return: The signed in :class:`User`.

        """
        User = get_user_model()
        try:
            django.core.validators.validate_email(identification)
            try:
                user = User.objects.get(email__iexact=identification)
            except User.DoesNotExist:
                return None
        except django.core.validators.ValidationError:
            try:
                user = User.objects.get(username__iexact=identification)
            except User.DoesNotExist:
                return None
        if check_password:
            if user.check_password(password):
                return user
            return None
        else:
            return user

    def get_user(self, user_id):
        User = get_user_model()
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None



from django.db import models
from django.db.models import Q

from userena.contrib.umessages import signals

import datetime


class MessageContactManager(models.Manager):
    """ Manager for the :class:`MessageContact` model """

    def get_or_create(self, um_from_user, um_to_user, message):
        """
        Get or create a Contact

        We override Django's :func:`get_or_create` because we want contact to
        be unique in a bi-directional manner.

        """
        created = False
        try:
            contact = self.get(
                Q(um_from_user=um_from_user, um_to_user=um_to_user)
                | Q(um_from_user=um_to_user, um_to_user=um_from_user)
            )

        except self.model.DoesNotExist:
            created = True
            contact = self.create(
                um_from_user=um_from_user, um_to_user=um_to_user, latest_message=message
            )

        return (contact, created)

    def update_contact(self, um_from_user, um_to_user, message):
        """ Get or update a contacts information """
        contact, created = self.get_or_create(um_from_user, um_to_user, message)

        # If the contact already existed, update the message
        if not created:
            contact.latest_message = message
            contact.save()
        return contact

    def get_contacts_for(self, user):
        """
        Returns the contacts for this user.

        Contacts are other users that this user has received messages
        from or send messages to.

        :param user:
            The :class:`User` which to get the contacts for.

        """
        contacts = self.filter(Q(um_from_user=user) | Q(um_to_user=user))
        return contacts


class MessageManager(models.Manager):
    """ Manager for the :class:`Message` model. """

    def send_message(self, sender, um_to_user_list, body):
        """
        Send a message from a user, to a user.

        :param sender:
            The :class:`User` which sends the message.

        :param um_to_user_list:
            A list which elements are :class:`User` to whom the message is for.

        :param message:
            String containing the message.

        """
        msg = self.model(sender=sender, body=body)
        msg.save()

        # Save the recipients
        msg.save_recipients(um_to_user_list)
        msg.update_contacts(um_to_user_list)
        signals.email_sent.send(sender=None, msg=msg)

        return msg

    def get_conversation_between(self, um_from_user, um_to_user):
        """ Returns a conversation between two users """
        messages = self.filter(
            Q(
                sender=um_from_user,
                recipients=um_to_user,
                sender_deleted_at__isnull=True,
            )
            | Q(
                sender=um_to_user,
                recipients=um_from_user,
                messagerecipient__deleted_at__isnull=True,
            )
        )
        return messages


class MessageRecipientManager(models.Manager):
    """ Manager for the :class:`MessageRecipient` model. """

    def count_unread_messages_for(self, user):
        """
        Returns the amount of unread messages for this user

        :param user:
            A Django :class:`User`

        :return:
            An integer with the amount of unread messages.

        """
        unread_total = self.filter(
            user=user, read_at__isnull=True, deleted_at__isnull=True
        ).count()

        return unread_total

    def count_unread_messages_between(self, um_to_user, um_from_user):
        """
        Returns the amount of unread messages between two users

        :param um_to_user:
            A Django :class:`User` for who the messages are for.

        :param um_from_user:
            A Django :class:`User` from whom the messages originate from.

        :return:
            An integer with the amount of unread messages.

        """
        unread_total = self.filter(
            message__sender=um_from_user,
            user=um_to_user,
            read_at__isnull=True,
            deleted_at__isnull=True,
        ).count()

        return unread_total



from django.test import TestCase
from django.urls import reverse, NoReverseMatch


class UserenaUrlsTests(TestCase):
    """ Test url resolve """

    def test_resolve_email_with_plus_url(self):
        username = "foo+bar@example.com"
        test_urls = [
            "userena_signup_complete",
            "userena_email_change",
            "userena_email_change_complete",
            "userena_email_confirm_complete",
            "userena_disabled",
            "userena_password_change",
            "userena_password_change_complete",
            "userena_profile_edit",
            "userena_profile_detail",
        ]
        for url_name in test_urls:
            try:
                reverse(url_name, kwargs={"username": username})
            except NoReverseMatch as ex:
                self.failed(ex)



import random
from collections import OrderedDict
from hashlib import sha1

from django import forms
from django.contrib.auth import authenticate
from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _

from userena import settings as userena_settings
from userena.models import UserenaSignup
from userena.utils import get_profile_model

attrs_dict = {"class": "required"}

USERNAME_RE = r"^[\.\w]+$"


class SignupForm(forms.Form):
    """
    Form for creating a new user account.

    Validates that the requested username and e-mail is not already in use.
    Also requires the password to be entered twice.
    """

    username = forms.RegexField(
        regex=USERNAME_RE,
        max_length=30,
        widget=forms.TextInput(attrs=attrs_dict),
        label=_("Username"),
        error_messages={
            "invalid": _(
                "Username must contain only letters, numbers, dots and underscores."
            )
        },
    )
    email = forms.EmailField(
        widget=forms.TextInput(attrs=dict(attrs_dict, maxlength=75)), label=_("Email")
    )
    password1 = forms.CharField(
        widget=forms.PasswordInput(attrs=attrs_dict, render_value=False),
        label=_("Create password"),
    )
    password2 = forms.CharField(
        widget=forms.PasswordInput(attrs=attrs_dict, render_value=False),
        label=_("Repeat password"),
    )

    def clean_username(self):
        """
        Validate that the username is alphanumeric and is not already in use.
        Also validates that the username is not listed in
        ``USERENA_FORBIDDEN_USERNAMES`` list.

        """
        try:
            user = get_user_model().objects.get(
                username__iexact=self.cleaned_data["username"]
            )
        except get_user_model().DoesNotExist:
            pass
        else:
            if userena_settings.USERENA_ACTIVATION_REQUIRED and UserenaSignup.objects.filter(
                user__username__iexact=self.cleaned_data["username"]
            ).exclude(
                activation_key=userena_settings.USERENA_ACTIVATED
            ):
                raise forms.ValidationError(
                    _(
                        "This username is already taken but not confirmed. Please check your email for verification steps."
                    )
                )
            raise forms.ValidationError(_("This username is already taken."))
        if (
            self.cleaned_data["username"].lower()
            in userena_settings.USERENA_FORBIDDEN_USERNAMES
        ):
            raise forms.ValidationError(_("This username is not allowed."))
        return self.cleaned_data["username"]

    def clean_email(self):
        """ Validate that the e-mail address is unique. """
        if get_user_model().objects.filter(email__iexact=self.cleaned_data["email"]):
            if userena_settings.USERENA_ACTIVATION_REQUIRED and UserenaSignup.objects.filter(
                user__email__iexact=self.cleaned_data["email"]
            ).exclude(
                activation_key=userena_settings.USERENA_ACTIVATED
            ):
                raise forms.ValidationError(
                    _(
                        "This email is already in use but not confirmed. Please check your email for verification steps."
                    )
                )
            raise forms.ValidationError(
                _("This email is already in use. Please supply a different email.")
            )
        return self.cleaned_data["email"]

    def clean(self):
        """
        Validates that the values entered into the two password fields match.
        Note that an error here will end up in ``non_field_errors()`` because
        it doesn't apply to a single field.

        """
        if "password1" in self.cleaned_data and "password2" in self.cleaned_data:
            if self.cleaned_data["password1"] != self.cleaned_data["password2"]:
                raise forms.ValidationError(_("The two password fields didn't match."))
        return self.cleaned_data

    def save(self):
        """ Creates a new user and account. Returns the newly created user. """
        username, email, password = (
            self.cleaned_data["username"],
            self.cleaned_data["email"],
            self.cleaned_data["password1"],
        )

        new_user = UserenaSignup.objects.create_user(
            username,
            email,
            password,
            not userena_settings.USERENA_ACTIVATION_REQUIRED,
            userena_settings.USERENA_ACTIVATION_REQUIRED,
        )
        return new_user


class SignupFormOnlyEmail(SignupForm):
    """
    Form for creating a new user account but not needing a username.

    This form is an adaptation of :class:`SignupForm`. It's used when
    ``USERENA_WITHOUT_USERNAME`` setting is set to ``True``. And thus the user
    is not asked to supply an username, but one is generated for them. The user
    can than keep sign in by using their email.

    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        del self.fields["username"]

    def save(self):
        """ Generate a random username before falling back to parent signup form """
        while True:
            username = sha1(str(random.random()).encode("utf-8")).hexdigest()[:5]
            try:
                get_user_model().objects.get(username__iexact=username)
            except get_user_model().DoesNotExist:
                break

        self.cleaned_data["username"] = username
        return super().save()


class SignupFormTos(SignupForm):
    """ Add a Terms of Service button to the ``SignupForm``. """

    tos = forms.BooleanField(
        widget=forms.CheckboxInput(attrs=attrs_dict),
        label=_("I have read and agree to the Terms of Service"),
        error_messages={"required": _("You must agree to the terms to register.")},
    )


def identification_field_factory(label, error_required):
    """
    A simple identification field factory which enable you to set the label.

    :param label:
        String containing the label for this field.

    :param error_required:
        String containing the error message if the field is left empty.

    """
    return forms.CharField(
        label=label,
        widget=forms.TextInput(attrs=attrs_dict),
        max_length=75,
        error_messages={"required": error_required},
    )


class AuthenticationForm(forms.Form):
    """
    A custom form where the identification can be a e-mail address or username.

    """

    identification = identification_field_factory(
        _("Email or username"), _("Either supply us with your email or username.")
    )
    password = forms.CharField(
        label=_("Password"),
        widget=forms.PasswordInput(attrs=attrs_dict, render_value=False),
    )
    remember_me = forms.BooleanField(
        widget=forms.CheckboxInput(),
        required=False,
        label=_("Remember me for %(days)s")
        % {"days": _(userena_settings.USERENA_REMEMBER_ME_DAYS[0])},
    )

    def __init__(self, *args, **kwargs):
        """ A custom init because we need to change the label if no usernames is used """
        super().__init__(*args, **kwargs)
        # Dirty hack, somehow the label doesn't get translated without declaring
        # it again here.
        self.fields["remember_me"].label = _("Remember me for %(days)s") % {
            "days": _(userena_settings.USERENA_REMEMBER_ME_DAYS[0])
        }
        if userena_settings.USERENA_WITHOUT_USERNAMES:
            self.fields["identification"] = identification_field_factory(
                _("Email"), _("Please supply your email.")
            )

    def clean(self):
        """
        Checks for the identification and password.

        If the combination can't be found will raise an invalid sign in error.

        """
        identification = self.cleaned_data.get("identification")
        password = self.cleaned_data.get("password")

        if identification and password:
            user = authenticate(identification=identification, password=password)
            if user is None:
                raise forms.ValidationError(
                    _(
                        "Please enter a correct username or email and password. Note that both fields are case-sensitive."
                    )
                )
        return self.cleaned_data


class ChangeEmailForm(forms.Form):
    email = forms.EmailField(
        widget=forms.TextInput(attrs=dict(attrs_dict, maxlength=75)),
        label=_("New email"),
    )

    def __init__(self, user, *args, **kwargs):
        """
        The current ``user`` is needed for initialisation of this form so
        that we can check if the email address is still free and not always
        returning ``True`` for this query because it's the users own e-mail
        address.

        """
        super().__init__(*args, **kwargs)
        if not isinstance(user, get_user_model()):
            raise TypeError(
                "user must be an instance of %s" % get_user_model().__name__
            )
        else:
            self.user = user

    def clean_email(self):
        """ Validate that the email is not already registered with another user """
        if self.cleaned_data["email"].lower() == self.user.email:
            raise forms.ValidationError(_("You're already known under this email."))
        if (
            get_user_model()
            .objects.filter(email__iexact=self.cleaned_data["email"])
            .exclude(email__iexact=self.user.email)
        ):
            raise forms.ValidationError(
                _("This email is already in use. Please supply a different email.")
            )
        return self.cleaned_data["email"]

    def save(self):
        """
        Save method calls :func:`user.change_email()` method which sends out an
        email with an verification key to verify and with it enable this new
        email address.

        """
        return self.user.userena_signup.change_email(self.cleaned_data["email"])


class EditProfileForm(forms.ModelForm):
    """ Base form used for fields that are always required """

    first_name = forms.CharField(label=_("First name"), max_length=30, required=False)
    last_name = forms.CharField(label=_("Last name"), max_length=30, required=False)

    def __init__(self, *args, **kw):
        super().__init__(*args, **kw)
        # Put the first and last name at the top
        new_order = [
            ("first_name", self.fields["first_name"]),
            ("last_name", self.fields["last_name"]),
        ]
        new_order.extend(list(self.fields.items())[:-2])
        self.fields = OrderedDict(new_order)

    class Meta:
        model = get_profile_model()
        exclude = ["user"]

    def save(self, force_insert=False, force_update=False, commit=True):
        profile = super().save(commit=commit)
        # Save first and last name
        user = profile.user
        user.first_name = self.cleaned_data["first_name"]
        user.last_name = self.cleaned_data["last_name"]
        user.save()

        return profile

class ActivationForm(forms.Form):
    """Form for activating an account."""
    pass



from django.conf import settings
from django.conf.urls import include
from django.conf.urls.static import static
from django.contrib import admin
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.urls import re_path

admin.autodiscover()

urlpatterns = [
    re_path(r"^admin/doc/", include("django.contrib.admindocs.urls")),
    re_path(r"^admin/", admin.site.urls),
    # Demo Override the signup form with our own, which includes a
    # first and last name.
    # (r'^accounts/signup/$',
    #  'userena.views.signup',
    #  {'signup_form': SignupFormExtra}),
    re_path(r"^accounts/", include("userena.urls")),
    re_path(r"^messages/", include("userena.contrib.umessages.urls")),
    re_path(r"^i18n/", include("django.conf.urls.i18n")),
]

# Add media and static files
urlpatterns += staticfiles_urlpatterns()
urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



from django.contrib.auth import get_user_model
from django.test import TestCase

from userena.contrib.umessages.forms import ComposeForm


class ComposeFormTests(TestCase):
    """ Test the compose form. """

    fixtures = ["users"]

    def test_invalid_data(self):
        """
        Test the save method of :class:`ComposeForm`

        We don't need to make the ``to`` field sweat because we have done that
        in the ``fields`` test.

        """
        invalid_data_dicts = [
            # No body
            {
                "data": {"to": "john", "body": ""},
                "error": ("body", ["This field is required."]),
            }
        ]

        for invalid_dict in invalid_data_dicts:
            form = ComposeForm(data=invalid_dict["data"])
            self.assertFalse(form.is_valid())
            self.assertEqual(
                form.errors[invalid_dict["error"][0]], invalid_dict["error"][1]
            )

    def test_save_msg(self):
        """ Test valid data """
        valid_data = {"to": "john, jane", "body": "Body"}

        form = ComposeForm(data=valid_data)

        self.assertTrue(form.is_valid())

        # Save the form.
        sender = get_user_model().objects.get(username="jane")
        msg = form.save(sender)

        # Check if the values are set correctly
        self.assertEqual(msg.body, valid_data["body"])
        self.assertEqual(msg.sender, sender)
        self.assertTrue(msg.sent_at)

        # Check recipients
        self.assertEqual(msg.recipients.all()[0].username, "jane")
        self.assertEqual(msg.recipients.all()[1].username, "john")



from django.db import models, migrations
import userena.managers


class Migration(migrations.Migration):

    dependencies = [("userena", "0001_initial")]

    operations = [
        migrations.AlterModelManagers(
            name="userenasignup",
            managers=[("objects", userena.managers.UserenaManager())],
        ),
        migrations.AlterField(
            model_name="userenasignup",
            name="email_unconfirmed",
            field=models.EmailField(
                help_text="Temporary email address when the user requests an email change.",
                max_length=254,
                verbose_name="unconfirmed email address",
                blank=True,
            ),
        ),
    ]



from django.contrib.auth import get_user_model
from django.http import HttpRequest
from django.test import TestCase

from userena.tests.profiles.models import Profile
from userena.middleware import UserenaLocaleMiddleware
from userena import settings as userena_settings
from userena.utils import get_user_profile, get_profile_model

User = get_user_model()


def has_profile(user):
    """Test utility function to check if user has profile"""
    profile_model = get_profile_model()
    try:
        profile = user.get_profile()
    except AttributeError:
        related_name = profile_model._meta.get_field("user").related_query_name()
        profile = getattr(user, related_name, None)
    except profile_model.DoesNotExist:
        profile = None

    return bool(profile)


class UserenaLocaleMiddlewareTests(TestCase):
    """ Test the ``UserenaLocaleMiddleware`` """

    fixtures = ["users", "profiles"]

    def _get_request_with_user(self, user):
        """ Fake a request with an user """
        request = HttpRequest()
        request.META = {"SERVER_NAME": "testserver", "SERVER_PORT": 80}
        request.method = "GET"
        request.session = {}

        # Add user
        request.user = user
        return request

    def test_preference_user(self):
        """ Test the language preference of two users """
        users = ((1, "nl"), (2, "en"))

        for pk, lang in users:
            user = User.objects.get(pk=pk)
            profile = get_user_profile(user=user)

            req = self._get_request_with_user(user)

            # Check that the user has this preference
            self.assertEqual(profile.language, lang)

            # Request should have a ``LANGUAGE_CODE`` with dutch
            UserenaLocaleMiddleware().process_request(req)
            self.assertEqual(req.LANGUAGE_CODE, lang)

    def test_without_profile(self):
        """ Middleware should do nothing when a user has no profile """
        # Delete the profile
        Profile.objects.get(pk=1).delete()
        user = User.objects.get(pk=1)

        # User shouldn't have a profile
        self.assertFalse(has_profile(user))

        req = self._get_request_with_user(user)
        UserenaLocaleMiddleware().process_request(req)

        self.assertFalse(hasattr(req, "LANGUAGE_CODE"))

    def test_without_language_field(self):
        """ Middleware should do nothing if the profile has no language field """
        userena_settings.USERENA_LANGUAGE_FIELD = "non_existant_language_field"
        user = User.objects.get(pk=1)

        req = self._get_request_with_user(user)

        # Middleware should do nothing
        UserenaLocaleMiddleware().process_request(req)
        self.assertFalse(hasattr(req, "LANGUAGE_CODE"))



from django.test import TestCase
from django.urls import reverse

from userena.tests.profiles.models import Profile


class PrivacyTests(TestCase):
    """
    Privacy testing of views concerning profiles.

    Test the privacy of the views that are available with three type of users:

        - Anonymous: An user that is not signed in.
        - Registered: An user that is registered and signed in.
        - Superuser: An user that is administrator at the site.

    """

    fixtures = ["users", "profiles"]

    reg_user = {"username": "jane", "password": "blowfish"}

    super_user = {"username": "john", "password": "blowfish"}

    detail_profile_url = reverse("userena_profile_detail", kwargs={"username": "john"})

    edit_profile_url = reverse("userena_profile_edit", kwargs={"username": "john"})

    def _test_status_codes(self, url, users_status):
        """
        Test if the status codes are corresponding to what that user should
        see.

        """
        for user, status in users_status:
            if user:
                self.client.login(**user)
            response = self.client.get(url, follow=True)
            self.assertEqual(response.status_code, status)

    def test_detail_open_profile_view(self):
        """ Viewing an open profile should be visible to everyone """
        profile = Profile.objects.get(pk=1)
        profile.privacy = "open"
        profile.save()

        users_status = ((None, 200), (self.reg_user, 200), (self.super_user, 200))
        self._test_status_codes(self.detail_profile_url, users_status)

    def test_detail_registered_profile_view(self):
        """ Viewing a users who's privacy is registered """
        profile = Profile.objects.get(pk=1)
        profile.privacy = "registered"
        profile.save()

        users_status = ((None, 403), (self.reg_user, 200), (self.super_user, 200))
        self._test_status_codes(self.detail_profile_url, users_status)

    def test_detail_closed_profile_view(self):
        """ Viewing a closed profile should only by visible to the admin """
        profile = Profile.objects.get(pk=1)
        profile.privacy = "closed"
        profile.save()

        users_status = ((None, 403), (self.reg_user, 403), (self.super_user, 200))
        self._test_status_codes(self.detail_profile_url, users_status)

    def test_edit_profile_view(self):
        """ Editing a profile should only be available to the owner and the admin """
        profile = Profile.objects.get(pk=1)

        users_status = ((None, 403), (self.reg_user, 403), (self.super_user, 200))
        self._test_status_codes(self.edit_profile_url, users_status)



from django.core.management.base import BaseCommand

from userena.models import UserenaSignup


class Command(BaseCommand):
    """
    Search for users that still haven't verified their email after
    ``USERENA_ACTIVATION_DAYS`` and delete them.

    """

    help = "Deletes expired users."

    def handle(self, *args, **kwargs):
        users = UserenaSignup.objects.delete_expired_users()



#!/usr/bin/python
from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.sites import shortcuts
from django.db import models
from django.utils.translation import gettext_lazy as _
from easy_thumbnails.fields import ThumbnailerImageField
from guardian.shortcuts import get_perms
from userena import settings as userena_settings
from userena.managers import UserenaManager, UserenaBaseProfileManager
from userena.utils import (
    get_gravatar,
    generate_nonce,
    get_protocol,
    get_datetime_now,
    user_model_label,
)
import datetime
from .mail import UserenaConfirmationMail


PROFILE_PERMISSIONS = (("view_profile", "Can view profile"),)


def upload_to_mugshot(instance, filename):
    """
    Uploads a mugshot for a user to the ``USERENA_MUGSHOT_PATH`` and saving it
    under unique hash for the image. This is for privacy reasons so others
    can't just browse through the mugshot directory.

    """
    extension = filename.split(".")[-1].lower()
    path = userena_settings.USERENA_MUGSHOT_PATH % {
        "username": instance.user.username,
        "id": instance.user.id,
        "date": instance.user.date_joined,
        "date_now": get_datetime_now().date(),
    }
    return "%(path)s%(hash)s.%(extension)s" % {
        "path": path,
        "hash": generate_nonce()[:10],
        "extension": extension,
    }


class UserenaSignup(models.Model):
    """
    Userena model which stores all the necessary information to have a full
    functional user implementation on your Django website.

    """

    user = models.OneToOneField(
        user_model_label,
        verbose_name=_("user"),
        related_name="userena_signup",
        on_delete=models.CASCADE,
    )

    last_active = models.DateTimeField(
        _("last active"),
        blank=True,
        null=True,
        help_text=_("The last date that the user was active."),
    )

    activation_key = models.CharField(_("activation key"), max_length=40, blank=True)

    activation_notification_send = models.BooleanField(
        _("notification send"),
        default=False,
        help_text=_(
            "Designates whether this user has already got a notification about activating their account."
        ),
    )

    email_unconfirmed = models.EmailField(
        _("unconfirmed email address"),
        blank=True,
        help_text=_("Temporary email address when the user requests an email change."),
    )

    email_confirmation_key = models.CharField(
        _("unconfirmed email verification key"), max_length=40, blank=True
    )

    email_confirmation_key_created = models.DateTimeField(
        _("creation date of email confirmation key"), blank=True, null=True
    )

    objects = UserenaManager()

    class Meta:
        verbose_name = _("userena registration")
        verbose_name_plural = _("userena registrations")

    def __str__(self):
        return "%s" % self.user.username

    def change_email(self, email):
        """
        Changes the email address for a user.

        A user needs to verify this new email address before it becomes
        active. By storing the new email address in a temporary field --
        ``temporary_email`` -- we are able to set this email address after the
        user has verified it by clicking on the verification URI in the email.
        This email gets send out by ``send_verification_email``.

        :param email:
            The new email address that the user wants to use.

        """
        self.email_unconfirmed = email

        self.email_confirmation_key = generate_nonce()
        self.email_confirmation_key_created = get_datetime_now()
        self.save()

        # Send email for activation
        self.send_confirmation_email()

    def send_confirmation_email(self):
        """
        Sends an email to confirm the new email address.

        This method sends out two emails. One to the new email address that
        contains the ``email_confirmation_key`` which is used to verify this
        this email address with :func:`UserenaUser.objects.confirm_email`.

        The other email is to the old email address to let the user know that
        a request is made to change this email address.

        """
        context = {
            "user": self.user,
            "without_usernames": userena_settings.USERENA_WITHOUT_USERNAMES,
            "new_email": self.email_unconfirmed,
            "protocol": get_protocol(),
            "confirmation_key": self.email_confirmation_key,
            "site": shortcuts.get_current_site(request=None),
        }

        mailer = UserenaConfirmationMail(context=context)
        mailer.generate_mail("confirmation", "_old")

        if self.user.email:
            mailer.send_mail(self.user.email)

        mailer.generate_mail("confirmation", "_new")
        mailer.send_mail(self.email_unconfirmed)

    @property
    def activation_completed(self):
        return self.activation_key == userena_settings.USERENA_ACTIVATED

    def activation_key_expired(self):
        """
        Checks if activation key is expired.

        Returns ``True`` when the ``activation_key`` of the user is expired and
        ``False`` if the key is still valid.

        The key is expired when it's set to the value defined in
        ``USERENA_ACTIVATED`` or ``activation_key_created`` is beyond the
        amount of days defined in ``USERENA_ACTIVATION_DAYS``.

        """
        expiration_days = datetime.timedelta(
            days=userena_settings.USERENA_ACTIVATION_DAYS
        )
        expiration_date = self.user.date_joined + expiration_days
        if self.activation_completed or (get_datetime_now() >= expiration_date):
            return True
        return False

    def send_activation_email(self):
        """
        Sends a activation email to the user.

        This email is send when the user wants to activate their newly created
        user.

        """
        context = {
            "user": self.user,
            "without_usernames": userena_settings.USERENA_WITHOUT_USERNAMES,
            "protocol": get_protocol(),
            "activation_days": userena_settings.USERENA_ACTIVATION_DAYS,
            "activation_key": self.activation_key,
            "site": shortcuts.get_current_site(request=None),
        }

        mailer = UserenaConfirmationMail(context=context)
        mailer.generate_mail("activation")
        mailer.send_mail(self.user.email)


class UserenaBaseProfile(models.Model):
    """ Base model needed for extra profile functionality """

    PRIVACY_CHOICES = (
        ("open", _("Open")),
        ("registered", _("Registered")),
        ("closed", _("Closed")),
    )

    MUGSHOT_SETTINGS = {
        "size": (
            userena_settings.USERENA_MUGSHOT_SIZE,
            userena_settings.USERENA_MUGSHOT_SIZE,
        ),
        "crop": userena_settings.USERENA_MUGSHOT_CROP_TYPE,
    }

    mugshot = ThumbnailerImageField(
        _("mugshot"),
        blank=True,
        upload_to=upload_to_mugshot,
        resize_source=MUGSHOT_SETTINGS,
        help_text=_("A personal image displayed in your profile."),
    )

    privacy = models.CharField(
        _("privacy"),
        max_length=15,
        choices=PRIVACY_CHOICES,
        default=userena_settings.USERENA_DEFAULT_PRIVACY,
        help_text=_("Designates who can view your profile."),
    )

    objects = UserenaBaseProfileManager()

    class Meta:
        """
        Meta options making the model abstract and defining permissions.

        The model is ``abstract`` because it only supplies basic functionality
        to a more custom defined model that extends it. This way there is not
        another join needed.

        We also define custom permissions because we don't know how the model
        that extends this one is going to be called. So we don't know what
        permissions to check. For ex. if the user defines a profile model that
        is called ``MyProfile``, than the permissions would be
        ``add_myprofile`` etc. We want to be able to always check
        ``add_profile``, ``change_profile`` etc.

        """

        abstract = True
        default_permissions = ("add", "change", "delete")
        permissions = PROFILE_PERMISSIONS

    def __str__(self):
        return "Profile of %(username)s" % {"username": self.user.username}

    def get_mugshot_url(self):
        """
        Returns the image containing the mugshot for the user.

        The mugshot can be a uploaded image or a Gravatar.

        Gravatar functionality will only be used when
        ``USERENA_MUGSHOT_GRAVATAR`` is set to ``True``.

        :return:
            ``None`` when Gravatar is not used and no default image is supplied
            by ``USERENA_MUGSHOT_DEFAULT``.

        """
        # First check for a mugshot and if any return that.
        if self.mugshot:
            return self.mugshot.url

        # Use Gravatar if the user wants to.
        if userena_settings.USERENA_MUGSHOT_GRAVATAR:
            return get_gravatar(
                self.user.email,
                userena_settings.USERENA_MUGSHOT_SIZE,
                userena_settings.USERENA_MUGSHOT_DEFAULT,
            )

        # Gravatar not used, check for a default image.
        else:
            if userena_settings.USERENA_MUGSHOT_DEFAULT not in [
                "404",
                "mm",
                "identicon",
                "monsterid",
                "wavatar",
            ]:
                return userena_settings.USERENA_MUGSHOT_DEFAULT
            else:
                return None

    def get_full_name_or_username(self):
        """
        Returns the full name of the user, or if none is supplied will return
        the username.

        Also looks at ``USERENA_WITHOUT_USERNAMES`` settings to define if it
        should return the username or email address when the full name is not
        supplied.

        :return:
            ``String`` containing the full name of the user. If no name is
            supplied it will return the username or email address depending on
            the ``USERENA_WITHOUT_USERNAMES`` setting.

        """
        user = self.user
        if user.first_name or user.last_name:
            # We will return this as translated string. Maybe there are some
            # countries that first display the last name.
            name = _("%(first_name)s %(last_name)s") % {
                "first_name": user.first_name,
                "last_name": user.last_name,
            }
        else:
            # Fallback to the username if usernames are used
            if not userena_settings.USERENA_WITHOUT_USERNAMES:
                name = "%(username)s" % {"username": user.username}
            else:
                name = "%(email)s" % {"email": user.email}
        return name.strip()

    def can_view_profile(self, user):
        """
        Can the :class:`User` view this profile?

        Returns a boolean if a user has the rights to view the profile of this
        user.

        Users are divided into four groups:

            ``Open``
                Everyone can view your profile

            ``Closed``
                Nobody can view your profile.

            ``Registered``
                Users that are registered on the website and signed
                in only.

            ``Admin``
                Special cases like superadmin and the owner of the profile.

        Through the ``privacy`` field a owner of an profile can define what
        they want to show to whom.

        :param user:
            A Django :class:`User` instance.

        """
        # Simple cases first, we don't want to waste CPU and DB hits.
        # Everyone.
        if self.privacy == "open":
            return True
        # Registered users.
        elif self.privacy == "registered" and isinstance(user, get_user_model()):
            return True

        # Checks done by guardian for owner and admins.
        elif "view_profile" in get_perms(user, self):
            return True

        # Fallback to closed profile.
        return False


class UserenaLanguageBaseProfile(UserenaBaseProfile):
    """
    Extends the :class:`UserenaBaseProfile` with a language choice.

    Use this model in combination with ``UserenaLocaleMiddleware`` automatically
    set the language of users when they are signed in.
    """

    # Max length is 8 to support language identifiers according to BCP47
    # https://tools.ietf.org/html/bcp47
    # The language identifier is of the form lll-Ssss
    #  - lll the language (shortest ISO 639 code, 3 letters max)
    #  - a hyphen
    #  - Ssss the script (ISO 15924 code, 4 letters)
    language = models.CharField(
        _("language"),
        max_length=8,
        choices=settings.LANGUAGES,
        default=settings.LANGUAGE_CODE[:2],
        help_text=_("Default language."),
    )

    class Meta:
        abstract = True
        default_permissions = ("add", "change", "delete")
        permissions = PROFILE_PERMISSIONS



from django import forms
from django.utils.translation import gettext_lazy as _

from userena.contrib.umessages.fields import CommaSeparatedUserField
from userena.contrib.umessages.models import Message, MessageRecipient

import datetime


class ComposeForm(forms.Form):
    to = CommaSeparatedUserField(label=_("To"))
    body = forms.CharField(
        label=_("Message"), widget=forms.Textarea({"class": "message"}), required=True
    )

    def save(self, sender):
        """
        Save the message and send it out into the wide world.

        :param sender:
            The :class:`User` that sends the message.

        :param parent_msg:
            The :class:`Message` that preceded this message in the thread.

        :return: The saved :class:`Message`.

        """
        um_to_user_list = self.cleaned_data["to"]
        body = self.cleaned_data["body"]

        msg = Message.objects.send_message(sender, um_to_user_list, body)

        return msg



import warnings

from django.contrib import messages
from django.contrib.auth import REDIRECT_FIELD_NAME, authenticate, login, logout
from django.contrib.auth import get_user_model
from django.contrib.auth.forms import PasswordChangeForm
from django.contrib.auth.views import LogoutView
from django.contrib.messages.views import SuccessMessageMixin
from django.core.exceptions import PermissionDenied
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse
from django.utils.decorators import method_decorator
from django.utils.translation import gettext as _
from django.views.generic import TemplateView
from django.views.generic.list import ListView
from guardian.decorators import permission_required_or_403

from userena import settings as userena_settings
from userena import signals as userena_signals
from userena.decorators import secure_required
from userena.forms import (
    ActivationForm,
    AuthenticationForm,
    ChangeEmailForm,
    EditProfileForm,
    SignupForm,
    SignupFormOnlyEmail,
)
from userena.models import UserenaSignup
from userena.utils import get_profile_model, get_user_profile, signin_redirect


class ExtraContextTemplateView(TemplateView):
    """ Add extra context to a simple template view """

    extra_context = None

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        if self.extra_context:
            context.update(self.extra_context)
        return context

    # this view is used in POST requests, e.g. signup when the form is not valid
    post = TemplateView.get


class ProfileListView(ListView):
    """ Lists all profiles """

    context_object_name = "profile_list"
    page = 1
    paginate_by = 50
    template_name = userena_settings.USERENA_PROFILE_LIST_TEMPLATE
    extra_context = None

    def get_context_data(self, **kwargs):
        # Call the base implementation first to get a context
        context = super().get_context_data(**kwargs)
        try:
            page = int(self.request.GET.get("page", None))
        except (TypeError, ValueError):
            page = self.page

        if (
            userena_settings.USERENA_DISABLE_PROFILE_LIST
            and not self.request.user.is_staff
        ):
            raise Http404

        if not self.extra_context:
            self.extra_context = dict()

        context["page"] = page
        context["paginate_by"] = self.paginate_by
        context["extra_context"] = self.extra_context

        return context

    def get_queryset(self):
        profile_model = get_profile_model()
        queryset = profile_model.objects.get_visible_profiles(
            self.request.user
        ).select_related()
        return queryset


@secure_required
def signup(
    request,
    signup_form=SignupForm,
    template_name="userena/signup_form.html",
    success_url=None,
    extra_context=None,
):
    """
    Signup of an account.

    Signup requiring a username, email and password. After signup a user gets
    an email with an activation link used to activate their account. After
    successful signup redirects to ``success_url``.

    :param signup_form:
        Form that will be used to sign a user. Defaults to userena's
        :class:`SignupForm`.

    :param template_name:
        String containing the template name that will be used to display the
        signup form. Defaults to ``userena/signup_form.html``.

    :param success_url:
        String containing the URI which should be redirected to after a
        successful signup. If not supplied will redirect to
        ``userena_signup_complete`` view.

    :param extra_context:
        Dictionary containing variables which are added to the template
        context. Defaults to a dictionary with a ``form`` key containing the
        ``signup_form``.

    **Context**

    ``form``
        Form supplied by ``signup_form``.

    """
    # If signup is disabled, return 403
    if userena_settings.USERENA_DISABLE_SIGNUP:
        raise PermissionDenied

    # If no usernames are wanted and the default form is used, fallback to the
    # default form that doesn't display to enter the username.
    if userena_settings.USERENA_WITHOUT_USERNAMES and (signup_form == SignupForm):
        signup_form = SignupFormOnlyEmail

    form = signup_form()

    if request.method == "POST":
        form = signup_form(request.POST, request.FILES)
        if form.is_valid():
            user = form.save()

            # Send the signup complete signal
            userena_signals.signup_complete.send(sender=None, user=user)

            if success_url:
                redirect_to = success_url
            else:
                redirect_to = reverse(
                    "userena_signup_complete", kwargs={"username": user.username}
                )

            # A new signed user should logout the old one.
            authenticated = request.user.is_authenticated

            if authenticated:
                logout(request)

            if (
                userena_settings.USERENA_SIGNIN_AFTER_SIGNUP
                and not userena_settings.USERENA_ACTIVATION_REQUIRED
            ):
                user = authenticate(identification=user.email, check_password=False)
                login(request, user)

            return redirect(redirect_to)

    if not extra_context:
        extra_context = dict()
    extra_context["form"] = form
    return ExtraContextTemplateView.as_view(
        template_name=template_name, extra_context=extra_context
    )(request)


@secure_required
def activate(
    request,
    activation_key,
    activation_form=ActivationForm,
    template_name="userena/activate_form.html",
    fail_template_name="userena/activate_fail.html",
    retry_template_name="userena/activate_retry.html",
    success_url=None,
    extra_context=None,
):
    """
    Activate a user with an activation key.

    The key is a nonce. When the nonce is found with an
    :class:`UserenaSignup`, the :class:`User` of that account will be
    activated.  After a successful activation the view will redirect to
    ``success_url``.  If the nonce is not found, the user will be shown the
    ``template_name`` template displaying a fail message.
    If the nonce is found but expired, ``retry_template_name`` is used instead,
    so the user can proceed to :func:`activate_retry` to get a new activation key.

    :param activation_key:
        Cryptographically generated string, 40 characters long

    :param activation_form:
        Form to use for activating the user. Defaults to
        :class:`ActivationForm` supplied by userena.

    :param template_name:
        String defining the name of the template to use. Defaults to
        ``userena/activate_form.html``.

    :param fail_template_name:
        String containing the template name that is used when the
        ``activation_key`` is invalid and the activation fails. Defaults to
        ``userena/activate_fail.html``.

    :param retry_template_name:
        String containing the template name that is used when the
        ``activation_key`` is expired. Defaults to
        ``userena/activate_retry.html``.

    :param success_url:
        String containing the URL where the user should be redirected to after
        a successful activation. Will replace ``%(username)s`` with string
        formatting if supplied. If ``success_url`` is left empty, will direct
        to ``userena_profile_detail`` view.

    :param extra_context:
        Dictionary containing variables which could be added to the template
        context. Default to an empty dictionary.

    """
    if not extra_context:
        extra_context = dict()

    form = activation_form()

    if request.method == "POST":
        form = activation_form(request.POST, request.FILES)
        if form.is_valid():
            try:
                if (
                    not UserenaSignup.objects.check_expired_activation(activation_key)
                    or not userena_settings.USERENA_ACTIVATION_RETRY
                ):
                    user = UserenaSignup.objects.activate_user(activation_key)
                    if user:
                        # Sign the user in.
                        auth_user = authenticate(
                            identification=user.email, check_password=False
                        )
                        login(request, auth_user)

                        if userena_settings.USERENA_USE_MESSAGES:
                            messages.success(
                                request,
                                _(
                                    "Your account has been activated and you have been signed in."
                                ),
                                fail_silently=True,
                            )

                        if success_url:
                            redirect_to = success_url % {"username": user.username}
                        else:
                            redirect_to = reverse(
                                "userena_profile_detail",
                                kwargs={"username": user.username},
                            )
                        return redirect(redirect_to)
                    else:
                        return ExtraContextTemplateView.as_view(
                            template_name=fail_template_name,
                            extra_context=extra_context,
                        )(request)
                else:
                    extra_context["activation_key"] = activation_key
                    return ExtraContextTemplateView.as_view(
                        template_name=retry_template_name, extra_context=extra_context
                    )(request)
            except UserenaSignup.DoesNotExist:
                return ExtraContextTemplateView.as_view(
                    template_name=fail_template_name, extra_context=extra_context
                )(request)

    extra_context.update({"form": form})
    return ExtraContextTemplateView.as_view(
        template_name=template_name, extra_context=extra_context
    )(request)


@secure_required
def activate_retry(
    request,
    activation_key,
    template_name="userena/activate_retry_success.html",
    extra_context=None,
):
    """
    Reissue a new ``activation_key`` for the user with the expired
    ``activation_key``.

    If ``activation_key`` does not exists, or ``USERENA_ACTIVATION_RETRY`` is
    set to False and for any other error condition user is redirected to
    :func:`activate` for error message display.

    :param activation_key:
        Cryptographically generated string, 40 characters long.

    :param template_name:
        String containing the template name that is used when new
        ``activation_key`` has been created. Defaults to
        ``userena/activate_retry_success.html``.

    :param extra_context:
        Dictionary containing variables which could be added to the template
        context. Default to an empty dictionary.

    """
    if not userena_settings.USERENA_ACTIVATION_RETRY:
        return redirect(reverse("userena_activate", args=(activation_key,)))
    try:
        if UserenaSignup.objects.check_expired_activation(activation_key):
            new_key = UserenaSignup.objects.reissue_activation(activation_key)
            if new_key:
                if not extra_context:
                    extra_context = dict()
                return ExtraContextTemplateView.as_view(
                    template_name=template_name, extra_context=extra_context
                )(request)
            else:
                return redirect(reverse("userena_activate", args=(activation_key,)))
        else:
            return redirect(reverse("userena_activate", args=(activation_key,)))
    except UserenaSignup.DoesNotExist:
        return redirect(reverse("userena_activate", args=(activation_key,)))


@secure_required
def email_confirm(
    request,
    confirmation_key,
    template_name="userena/email_confirm_fail.html",
    success_url=None,
    extra_context=None,
):
    """
    Confirms an email address with a confirmation key.

    Confirms a new email address by running :func:`User.objects.confirm_email`
    method. If the method returns an :class:`User` the user will have his new
    e-mail address set and redirected to ``success_url``. If no ``User`` is
    returned the user will be represented with a fail message from
    ``template_name``.

    :param confirmation_key:
        String with representing the confirmation key used to verify a
        new email address.

    :param template_name:
        String containing the template name which should be rendered when
        confirmation fails. When confirmation is successful, no template is
        needed because the user will be redirected to ``success_url``.

    :param success_url:
        String containing the URL which is redirected to after a successful
        confirmation.  Supplied argument must be able to be rendered by
        ``reverse`` function.

    :param extra_context:
        Dictionary of variables that are passed on to the template supplied by
        ``template_name``.

    """
    user = UserenaSignup.objects.confirm_email(confirmation_key)
    if user:
        if userena_settings.USERENA_USE_MESSAGES:
            messages.success(
                request, _("Your email address has been changed."), fail_silently=True
            )

        if success_url:
            redirect_to = success_url
        else:
            redirect_to = reverse(
                "userena_email_confirm_complete", kwargs={"username": user.username}
            )
        return redirect(redirect_to)
    else:
        if not extra_context:
            extra_context = dict()
        return ExtraContextTemplateView.as_view(
            template_name=template_name, extra_context=extra_context
        )(request)


def direct_to_user_template(request, username, template_name, extra_context=None):
    """
    Simple wrapper for Django's :func:`direct_to_template` view.

    This view is used when you want to show a template to a specific user. A
    wrapper for :func:`direct_to_template` where the template also has access to
    the user that is found with ``username``. For ex. used after signup,
    activation and confirmation of a new e-mail.

    :param username:
        String defining the username of the user that made the action.

    :param template_name:
        String defining the name of the template to use. Defaults to
        ``userena/signup_complete.html``.

    **Keyword arguments**

    ``extra_context``
        A dictionary containing extra variables that should be passed to the
        rendered template. The ``account`` key is always the ``User``
        that completed the action.

    **Extra context**

    ``viewed_user``
        The currently :class:`User` that is viewed.

    """
    user = get_object_or_404(get_user_model(), username__iexact=username)

    if not extra_context:
        extra_context = dict()
    extra_context["viewed_user"] = user
    extra_context["profile"] = get_user_profile(user=user)
    return ExtraContextTemplateView.as_view(
        template_name=template_name, extra_context=extra_context
    )(request)


def disabled_account(
    request, username, template_name="userena/disabled.html", extra_context=None
):
    """
    Checks if the account is not active, if so, returns the disabled
    account template.

    The ``disabled_account`` view has a high bar: it should only be
    shown if the user has a completed activation.  Otherwise,
    redirect to `userena_activate_pending``.

    If no ``UserenaSignup`` object can be found for the user, we will
    still assume that it was deleted after expiration but not that
    account was deliberately disabled.

    :param username:
        String defining the username of the user that made the action.

    :param template_name:
        String defining the name of the template to use. Defaults to
        ``userena/disabled.html``.

    **Keyword arguments**

    ``extra_context``
        A dictionary containing extra variables that should be passed to the
        rendered template. The ``account`` key is always the ``User``
        that completed the action.

    **Extra context**

    ``viewed_user``
        The currently :class:`User` that is viewed.

    ``profile``
        Profile of the viewed user.

    """
    user = get_object_or_404(
        get_user_model(), username__iexact=username, is_active=False
    )

    if not user.userena_signup.activation_completed:
        return redirect(
            reverse("userena_activate_pending", kwargs={"username": user.username})
        )

    if not extra_context:
        extra_context = dict()
    extra_context["viewed_user"] = user
    extra_context["profile"] = get_user_profile(user=user)
    return ExtraContextTemplateView.as_view(
        template_name=template_name, extra_context=extra_context
    )(request)


def activate_pending(
    request, username, template_name="userena/activate_pending.html", extra_context=None
):
    """
    Checks if the account is not active, if so, returns the
    activation pending template.  This view is meant to take
    precedent over the ``disabled_account`` view unless we know that the
    account was disabled after completion.

    :param username:
        String defining the username of the user that made the action.

    :param template_name:
        String defining the name of the template to use. Defaults to
        ``userena/activate_pending.html``.

    **Keyword arguments**

    ``extra_context``
        A dictionary containing extra variables that should be passed to the
        rendered template. The ``account`` key is always the ``User``
        that completed the action.

    """
    user = get_object_or_404(
        get_user_model(), username__iexact=username, is_active=False
    )

    # If we know that the activation process was completed, but the
    # user is now not active, it is safe to assume that the user was
    # actually disabled after completion of activation.  In that
    # case, we will redirect to ``userena_disabled``.
    if user.userena_signup.activation_completed:
        return redirect(reverse("userena_disabled", kwargs={"username": user.username}))

    if not extra_context:
        extra_context = dict()
    return ExtraContextTemplateView.as_view(
        template_name=template_name, extra_context=extra_context
    )(request)


@secure_required
def signin(
    request,
    auth_form=AuthenticationForm,
    template_name="userena/signin_form.html",
    redirect_field_name=REDIRECT_FIELD_NAME,
    redirect_signin_function=signin_redirect,
    extra_context=None,
):
    """
    Signin using email or username with password.

    Signs a user in by combining email/username with password. If the
    combination is correct and the user :func:`is_active` the
    :func:`redirect_signin_function` is called with the arguments
    ``REDIRECT_FIELD_NAME`` and an instance of the :class:`User` who is is
    trying the login. The returned value of the function will be the URL that
    is redirected to.

    A user can also select to be remembered for ``USERENA_REMEMBER_DAYS``.

    :param auth_form:
        Form to use for signing the user in. Defaults to the
        :class:`AuthenticationForm` supplied by userena.

    :param template_name:
        String defining the name of the template to use. Defaults to
        ``userena/signin_form.html``.

    :param redirect_field_name:
        Form field name which contains the value for a redirect to the
        succeeding page. Defaults to ``next`` and is set in
        ``REDIRECT_FIELD_NAME`` setting.

    :param redirect_signin_function:
        Function which handles the redirect. This functions gets the value of
        ``REDIRECT_FIELD_NAME`` and the :class:`User` who has logged in. It
        must return a string which specifies the URI to redirect to.

    :param extra_context:
        A dictionary containing extra variables that should be passed to the
        rendered template. The ``form`` key is always the ``auth_form``.

    **Context**

    ``form``
        Form used for authentication supplied by ``auth_form``.

    """
    form = auth_form()

    if request.method == "POST":
        form = auth_form(request.POST, request.FILES)
        if form.is_valid():
            identification, password, remember_me = (
                form.cleaned_data["identification"],
                form.cleaned_data["password"],
                form.cleaned_data["remember_me"],
            )
            user = authenticate(identification=identification, password=password)
            if user.is_active:
                login(request, user)
                if remember_me:
                    request.session.set_expiry(
                        userena_settings.USERENA_REMEMBER_ME_DAYS[1] * 86400
                    )
                else:
                    request.session.set_expiry(0)

                if userena_settings.USERENA_USE_MESSAGES:
                    messages.success(
                        request, _("You have been signed in."), fail_silently=True
                    )

                # send a signal that a user has signed in
                userena_signals.account_signin.send(sender=None, user=user)
                # Whereto now?
                redirect_to = redirect_signin_function(
                    request.GET.get(
                        redirect_field_name, request.POST.get(redirect_field_name)
                    ),
                    user,
                )
                return HttpResponseRedirect(redirect_to)
            else:
                # If the user is inactive, despite completing the
                # activation process, show the 'Account disabled'
                # page.  Otherwise, show the 'Activation pending'
                # page to encourage activation.
                if user.userena_signup.activation_completed:
                    return redirect(
                        reverse("userena_disabled", kwargs={"username": user.username})
                    )
                else:
                    return redirect(
                        reverse(
                            "userena_activate_pending",
                            kwargs={"username": user.username},
                        )
                    )

    if not extra_context:
        extra_context = dict()
    extra_context.update(
        {
            "form": form,
            "next": request.GET.get(
                redirect_field_name, request.POST.get(redirect_field_name)
            ),
        }
    )
    return ExtraContextTemplateView.as_view(
        template_name=template_name, extra_context=extra_context
    )(request)


class SignoutView(LogoutView, SuccessMessageMixin):
    """
    Signs out the user and adds a success message ``You have been signed
    out.`` If next_page is defined you will be redirected to the URI. If
    not the template in template_name is used.

    :param next_page:
        A string which specifies the URI to redirect to.

    :param template_name:
        String defining the name of the template to use. Defaults to
        ``userena/signout.html``.

    """

    template_name = "userena/signout.html"
    next_page = userena_settings.USERENA_REDIRECT_ON_SIGNOUT

    def get_success_message(self, cleaned_data):
        authenticated = self.request.user.is_authenticated

        if authenticated and userena_settings.USERENA_USE_MESSAGES:  # pragma: no cover
            return _("You have been signed out.")
        else:
            return ""

    @method_decorator(secure_required)
    def dispatch(self, request, *args, **kwargs):
        response = super().dispatch(request, *args, **kwargs)
        userena_signals.account_signout.send(sender=None, user=request.user)
        return response


@secure_required
@permission_required_or_403("change_user", (get_user_model(), "username", "username"))
def email_change(
    request,
    username,
    email_form=ChangeEmailForm,
    template_name="userena/email_form.html",
    success_url=None,
    extra_context=None,
):
    """
    Change email address

    :param username:
        String of the username which specifies the current account.

    :param email_form:
        Form that will be used to change the email address. Defaults to
        :class:`ChangeEmailForm` supplied by userena.

    :param template_name:
        String containing the template to be used to display the email form.
        Defaults to ``userena/email_form.html``.

    :param success_url:
        Named URL where the user will get redirected to when successfully
        changing their email address.  When not supplied will redirect to
        ``userena_email_complete`` URL.

    :param extra_context:
        Dictionary containing extra variables that can be used to render the
        template. The ``form`` key is always the form supplied by the keyword
        argument ``form`` and the ``user`` key by the user whose email address
        is being changed.

    **Context**

    ``form``
        Form that is used to change the email address supplied by ``form``.

    ``account``
        Instance of the ``Account`` whose email address is about to be changed.

    **Todo**

    Need to have per-object permissions, which enables users with the correct
    permissions to alter the email address of others.

    """
    user = get_object_or_404(get_user_model(), username__iexact=username)
    prev_email = user.email
    form = email_form(user)

    if request.method == "POST":
        form = email_form(user, request.POST, request.FILES)

        if form.is_valid():
            form.save()

            if success_url:
                # Send a signal that the email has changed
                userena_signals.email_change.send(
                    sender=None, user=user, prev_email=prev_email, new_email=user.email
                )
                redirect_to = success_url
            else:
                redirect_to = reverse(
                    "userena_email_change_complete", kwargs={"username": user.username}
                )
            return redirect(redirect_to)

    if not extra_context:
        extra_context = dict()
    extra_context["form"] = form
    extra_context["profile"] = get_user_profile(user=user)
    return ExtraContextTemplateView.as_view(
        template_name=template_name, extra_context=extra_context
    )(request)


@secure_required
@permission_required_or_403("change_user", (get_user_model(), "username", "username"))
def password_change(
    request,
    username,
    template_name="userena/password_form.html",
    pass_form=PasswordChangeForm,
    success_url=None,
    extra_context=None,
):
    """ Change password of user.

    This view is almost a mirror of the view supplied in
    :func:`contrib.auth.views.password_change`, with the minor change that in
    this view we also use the username to change the password. This was needed
    to keep our URLs logical (and REST) across the entire application. And
    that in a later stadium administrators can also change the users password
    through the web application itself.

    :param username:
        String supplying the username of the user who's password is about to be
        changed.

    :param template_name:
        String of the name of the template that is used to display the password
        change form. Defaults to ``userena/password_form.html``.

    :param pass_form:
        Form used to change password. Default is the form supplied by Django
        itself named ``PasswordChangeForm``.

    :param success_url:
        Named URL that is passed onto a :func:`reverse` function with
        ``username`` of the active user. Defaults to the
        ``userena_password_complete`` URL.

    :param extra_context:
        Dictionary of extra variables that are passed on to the template. The
        ``form`` key is always used by the form supplied by ``pass_form``.

    **Context**

    ``form``
        Form used to change the password.

    """
    user = get_object_or_404(get_user_model(), username__iexact=username)

    form = pass_form(user=user)

    if request.method == "POST":
        form = pass_form(user=user, data=request.POST)
        if form.is_valid():
            form.save()

            # Send a signal that the password has changed
            userena_signals.password_complete.send(sender=None, user=user)

            if success_url:
                redirect_to = success_url
            else:
                redirect_to = reverse(
                    "userena_password_change_complete",
                    kwargs={"username": user.username},
                )
            return redirect(redirect_to)

    if not extra_context:
        extra_context = dict()
    extra_context["form"] = form
    extra_context["profile"] = get_user_profile(user=user)
    return ExtraContextTemplateView.as_view(
        template_name=template_name, extra_context=extra_context
    )(request)


@secure_required
@permission_required_or_403(
    "change_profile", (get_profile_model(), "user__username", "username")
)
def profile_edit(
    request,
    username,
    edit_profile_form=EditProfileForm,
    template_name="userena/profile_form.html",
    success_url=None,
    extra_context=None,
    **kwargs
):
    """
    Edit profile.

    Edits a profile selected by the supplied username. First checks
    permissions if the user is allowed to edit this profile, if denied will
    show a 404. When the profile is successfully edited will redirect to
    ``success_url``.

    :param username:
        Username of the user which profile should be edited.

    :param edit_profile_form:

        Form that is used to edit the profile. The :func:`EditProfileForm.save`
        method of this form will be called when the form
        :func:`EditProfileForm.is_valid`.  Defaults to :class:`EditProfileForm`
        from userena.

    :param template_name:
        String of the template that is used to render this view. Defaults to
        ``userena/edit_profile_form.html``.

    :param success_url:
        Named URL which will be passed on to a django ``reverse`` function after
        the form is successfully saved. Defaults to the ``userena_detail`` url.

    :param extra_context:
        Dictionary containing variables that are passed on to the
        ``template_name`` template.  ``form`` key will always be the form used
        to edit the profile, and the ``profile`` key is always the edited
        profile.

    **Context**

    ``form``
        Form that is used to alter the profile.

    ``profile``
        Instance of the ``Profile`` that is edited.

    """
    user = get_object_or_404(get_user_model(), username__iexact=username)

    profile = get_user_profile(user=user)

    user_initial = {"first_name": user.first_name, "last_name": user.last_name}

    form = edit_profile_form(instance=profile, initial=user_initial)

    if request.method == "POST":
        form = edit_profile_form(
            request.POST, request.FILES, instance=profile, initial=user_initial
        )

        if form.is_valid():
            profile = form.save()

            if userena_settings.USERENA_USE_MESSAGES:
                messages.success(
                    request, _("Your profile has been updated."), fail_silently=True
                )

            if success_url:
                # Send a signal that the profile has changed
                userena_signals.profile_change.send(sender=None, user=user)
                redirect_to = success_url
            else:
                redirect_to = reverse(
                    "userena_profile_detail", kwargs={"username": username}
                )
            return redirect(redirect_to)

    if not extra_context:
        extra_context = dict()
    extra_context["form"] = form
    extra_context["profile"] = profile
    return ExtraContextTemplateView.as_view(
        template_name=template_name, extra_context=extra_context
    )(request)


def profile_detail(
    request,
    username,
    template_name=userena_settings.USERENA_PROFILE_DETAIL_TEMPLATE,
    extra_context=None,
    **kwargs
):
    """
    Detailed view of an user.

    :param username:
        String of the username of which the profile should be viewed.

    :param template_name:
        String representing the template name that should be used to display
        the profile.

    :param extra_context:
        Dictionary of variables which should be supplied to the template. The
        ``profile`` key is always the current profile.

    **Context**

    ``profile``
        Instance of the currently viewed ``Profile``.

    """
    user = get_object_or_404(get_user_model(), username__iexact=username)
    profile = get_user_profile(user=user)
    if not profile.can_view_profile(request.user):
        raise PermissionDenied
    if not extra_context:
        extra_context = dict()
    extra_context["profile"] = profile
    extra_context["hide_email"] = userena_settings.USERENA_HIDE_EMAIL
    return ExtraContextTemplateView.as_view(
        template_name=template_name, extra_context=extra_context
    )(request)


def profile_list(
    request,
    page=1,
    template_name="userena/profile_list.html",
    paginate_by=50,
    extra_context=None,
    **kwargs
):  # pragma: no cover
    """
    Returns a list of all profiles that are public.

    It's possible to disable this by changing ``USERENA_DISABLE_PROFILE_LIST``
    to ``True`` in your settings.

    :param page:
        Integer of the active page used for pagination. Defaults to the first
        page.

    :param template_name:
        String defining the name of the template that is used to render the
        list of all users. Defaults to ``userena/list.html``.

    :param paginate_by:
        Integer defining the amount of displayed profiles per page. Defaults to
        50 profiles per page.

    :param extra_context:
        Dictionary of variables that are passed on to the ``template_name``
        template.

    **Context**

    ``profile_list``
        A list of profiles.

    ``is_paginated``
        A boolean representing whether the results are paginated.

    If the result is paginated. It will also contain the following variables.

    ``paginator``
        An instance of ``django.core.paginator.Paginator``.

    ``page_obj``
        An instance of ``django.core.paginator.Page``.

    """
    warnings.warn(
        "views.profile_list is deprecated. Use ProfileListView instead",
        DeprecationWarning,
        stacklevel=2,
    )

    try:
        page = int(request.GET.get("page", None))
    except (TypeError, ValueError):
        page = page

    if userena_settings.USERENA_DISABLE_PROFILE_LIST and not request.user.is_staff:
        raise Http404

    profile_model = get_profile_model()
    queryset = profile_model.objects.get_visible_profiles(request.user)

    if not extra_context:
        extra_context = dict()
    return ProfileListView.as_view(
        queryset=queryset,
        paginate_by=paginate_by,
        page=page,
        template_name=template_name,
        extra_context=extra_context,
        **kwargs
    )(request)



from django.core.management.base import BaseCommand
from django.utils.encoding import smart_str

from userena.models import UserenaSignup

arguments = (
    (
        "--no-output",
        {
            "action": "store_false",
            "dest": "output",
            "default": True,
            "help": "Hide informational output.",
        },
    ),
    (
        "--test",
        {
            "action": "store_true",
            "dest": "test",
            "default": False,
            "help": "Displays that it's testing management command. Don't use it yourself.",
        },
    ),
)


class Command(BaseCommand):
    """
    For unknown reason, users can get wrong permissions.
    This command checks that all permissions are correct.

    """

    def add_arguments(self, parser):
        for arg, attrs in arguments:
            parser.add_argument(arg, **attrs)

    help = "Check that user permissions are correct."

    def handle(self, **options):
        permissions, users, warnings = UserenaSignup.objects.check_permissions()
        output = options.pop("output")
        test = options.pop("test")
        if test:
            self.stdout.write(40 * ".")
            self.stdout.write(
                "\nChecking permission management command. Ignore output..\n\n"
            )
        if output:
            for p in permissions:
                self.stdout.write("Added permission: %s\n" % p)

            for u in users:
                self.stdout.write(
                    "Changed permissions for user: %s\n"
                    % smart_str(u, encoding="utf-8", strings_only=False)
                )

            for w in warnings:
                self.stdout.write("WARNING: %s\n" % w)

        if test:
            self.stdout.write("\nFinished testing permissions command.. continuing..\n")



from django import forms
from django.utils.translation import gettext_lazy as _
from django.contrib import admin
from django.contrib.auth.models import Group

from userena.contrib.umessages.models import Message, MessageContact, MessageRecipient


class MessageRecipientInline(admin.TabularInline):
    """ Inline message recipients """

    model = MessageRecipient


class MessageAdmin(admin.ModelAdmin):
    """ Admin message class with inline recipients """

    inlines = [MessageRecipientInline]

    fieldsets = (
        (None, {"fields": ("sender", "body"), "classes": ("monospace")}),
        (
            _("Date/time"),
            {"fields": ("sender_deleted_at",), "classes": ("collapse", "wide")},
        ),
    )
    list_display = ("sender", "body", "sent_at")
    list_filter = ("sent_at", "sender")
    search_fields = ("body",)


admin.site.register(Message, MessageAdmin)
admin.site.register(MessageContact)



from django.contrib import messages
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth import get_user_model
from django.contrib.auth.decorators import login_required
from django.shortcuts import get_object_or_404, redirect, render
from django.urls import reverse
from django.utils.translation import gettext as _
from django.utils.translation import ngettext
from django.views.decorators.http import require_http_methods
from django.views.generic.list import ListView

from userena import settings as userena_settings
from userena.contrib.umessages.forms import ComposeForm
from userena.contrib.umessages.models import Message, MessageRecipient, MessageContact
from userena.utils import get_datetime_now


class MessageListView(ListView):
    """

    Returns the message list for this user. This is a list contacts
    which at the top has the user that the last conversation was with. This is
    an imitation of the iPhone SMS functionality.

    """

    page = 1
    paginate_by = 50
    template_name = "umessages/message_list.html"
    extra_context = {}
    context_object_name = "message_list"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(self.extra_context)
        return context

    def get_queryset(self):
        return MessageContact.objects.get_contacts_for(self.request.user)


class MessageDetailListView(MessageListView):
    """

    Returns a conversation between two users

    """

    template_name = "umessages/message_detail.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["recipient"] = self.recipient
        return context

    def get_queryset(self):
        username = self.kwargs["username"]
        self.recipient = get_object_or_404(get_user_model(), username__iexact=username)
        queryset = Message.objects.get_conversation_between(
            self.request.user, self.recipient
        )
        self._update_unread_messages(queryset)
        return queryset

    def _update_unread_messages(self, queryset):
        message_pks = [m.pk for m in queryset]
        unread_list = MessageRecipient.objects.filter(
            message__in=message_pks, user=self.request.user, read_at__isnull=True
        )
        now = get_datetime_now()
        unread_list.update(read_at=now)


@login_required
def message_compose(
    request,
    recipients=None,
    compose_form=ComposeForm,
    success_url=None,
    template_name="umessages/message_form.html",
    recipient_filter=None,
    extra_context=None,
):
    """
    Compose a new message

    :recipients:
        String containing the usernames to whom the message is send to. Can be
        multiple username by seperating them with a ``+`` sign.

    :param compose_form:
        The form that is used for getting neccesary information. Defaults to
        :class:`ComposeForm`.

    :param success_url:
        String containing the named url which to redirect to after successfull
        sending a message. Defaults to ``userena_umessages_list`` if there are
        multiple recipients. If there is only one recipient, will redirect to
        ``userena_umessages_detail`` page, showing the conversation.

    :param template_name:
        String containing the name of the template that is used.

    :param recipient_filter:
        A list of :class:`User` that don"t want to receive any messages.

    :param extra_context:
        Dictionary with extra variables supplied to the template.

    **Context**

    ``form``
        The form that is used.

    """
    initial_data = dict()

    if recipients:
        username_list = [r.strip() for r in recipients.split("+")]
        recipients = [
            u for u in get_user_model().objects.filter(username__in=username_list)
        ]
        initial_data["to"] = recipients

    form = compose_form(initial=initial_data)
    if request.method == "POST":
        form = compose_form(request.POST)
        if form.is_valid():
            requested_redirect = request.GET.get(
                REDIRECT_FIELD_NAME, request.POST.get(REDIRECT_FIELD_NAME, False)
            )

            message = form.save(request.user)
            recipients = form.cleaned_data["to"]

            if userena_settings.USERENA_USE_MESSAGES:
                messages.success(request, _("Message is sent."), fail_silently=True)

            # Redirect mechanism
            redirect_to = reverse("userena_umessages_list")
            if requested_redirect:
                redirect_to = requested_redirect
            elif success_url:
                redirect_to = success_url
            elif len(recipients) == 1:
                redirect_to = reverse(
                    "userena_umessages_detail",
                    kwargs={"username": recipients[0].username},
                )
            return redirect(redirect_to)

    if not extra_context:
        extra_context = dict()
    extra_context["form"] = form
    extra_context["recipients"] = recipients
    return render(request, template_name, extra_context)


@login_required
@require_http_methods(["POST"])
def message_remove(request, undo=False):
    """
    A ``POST`` to remove messages.

    :param undo:
        A Boolean that if ``True`` unremoves messages.

    POST can have the following keys:

        ``message_pks``
            List of message id's that should be deleted.

        ``next``
            String containing the URI which to redirect to after the keys are
            removed. Redirect defaults to the inbox view.

    The ``next`` value can also be supplied in the URI with ``?next=<value>``.

    """
    message_pks = request.POST.getlist("message_pks")
    redirect_to = request.GET.get(
        REDIRECT_FIELD_NAME, request.POST.get(REDIRECT_FIELD_NAME, False)
    )

    if message_pks:
        # Check that all values are integers.
        valid_message_pk_list = set()
        for pk in message_pks:
            try:
                valid_pk = int(pk)
            except (TypeError, ValueError):
                pass
            else:
                valid_message_pk_list.add(valid_pk)

        # Delete all the messages, if they belong to the user.
        now = get_datetime_now()
        changed_message_list = set()
        for pk in valid_message_pk_list:
            message = get_object_or_404(Message, pk=pk)

            # Check if the user is the owner
            if message.sender == request.user:
                if undo:
                    message.sender_deleted_at = None
                else:
                    message.sender_deleted_at = now
                message.save()
                changed_message_list.add(message.pk)

            # Check if the user is a recipient of the message
            if request.user in message.recipients.all():
                mr = message.messagerecipient_set.get(
                    user=request.user, message=message
                )
                if undo:
                    mr.deleted_at = None
                else:
                    mr.deleted_at = now
                mr.save()
                changed_message_list.add(message.pk)

        # Send messages
        if (len(changed_message_list) > 0) and userena_settings.USERENA_USE_MESSAGES:
            if undo:
                message = ngettext(
                    "Message is succesfully restored.",
                    "Messages are succesfully restored.",
                    len(changed_message_list),
                )
            else:
                message = ngettext(
                    "Message is successfully removed.",
                    "Messages are successfully removed.",
                    len(changed_message_list),
                )

            messages.success(request, message, fail_silently=True)

    if redirect_to:
        return redirect(redirect_to)
    else:
        return redirect(reverse("userena_umessages_list"))



from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.utils import translation
from django.utils.deprecation import MiddlewareMixin

from userena import settings as userena_settings
from userena.compat import SiteProfileNotAvailable
from userena.utils import get_user_profile


class UserenaLocaleMiddleware(MiddlewareMixin):
    """
    Set the language by looking at the language setting in the profile.

    It doesn't override the cookie that is set by Django so a user can still
    switch languages depending if the cookie is set.

    """
    def __init__(self, get_response=None):

        if get_response is None:
            # Fix RemovedInDjango40Warning: Passing None for the middleware
            # get_response argument is deprecated
            get_response = lambda r: r

        super().__init__(get_response=get_response)

    def process_request(self, request):
        lang_cookie = request.session.get(settings.LANGUAGE_COOKIE_NAME)
        if not lang_cookie:

            authenticated = request.user.is_authenticated

            if authenticated:
                try:
                    profile = get_user_profile(user=request.user)
                except (ObjectDoesNotExist, SiteProfileNotAvailable):
                    profile = False

                if profile:
                    try:
                        lang = getattr(profile, userena_settings.USERENA_LANGUAGE_FIELD)
                        translation.activate(lang)
                        request.LANGUAGE_CODE = translation.get_language()
                    except AttributeError:
                        pass



from django.db import models
from django.utils.translation import gettext_lazy as _

from userena.models import UserenaBaseProfile
from userena.utils import user_model_label


class Profile(UserenaBaseProfile):
    """ Default profile """

    GENDER_CHOICES = ((1, _("Male")), (2, _("Female")))

    user = models.OneToOneField(
        user_model_label,
        unique=True,
        verbose_name=_("user"),
        related_name="profile",
        on_delete=models.CASCADE,
    )

    gender = models.PositiveSmallIntegerField(
        _("gender"), choices=GENDER_CHOICES, blank=True, null=True
    )
    website = models.URLField(_("website"), blank=True)
    location = models.CharField(_("location"), max_length=255, blank=True)
    about_me = models.TextField(_("about me"), blank=True)
    language = models.TextField(_("language"), blank=True)

    class Meta(UserenaBaseProfile.Meta):
        ordering = ["user"]


class SecondProfile(UserenaBaseProfile):
    user = models.OneToOneField(
        user_model_label,
        unique=True,
        verbose_name=_("user"),
        related_name="profile_second",
        on_delete=models.CASCADE,
    )



from django.urls import re_path

from userena.contrib.umessages import views as messages_views
from django.contrib.auth.decorators import login_required

urlpatterns = [
    re_path(
        r"^compose/$", messages_views.message_compose, name="userena_umessages_compose"
    ),
    re_path(
        r"^compose/(?P<recipients>[\+\.\w]+)/$",
        messages_views.message_compose,
        name="userena_umessages_compose_to",
    ),
    re_path(
        r"^reply/(?P<parent_id>[\d]+)/$",
        messages_views.message_compose,
        name="userena_umessages_reply",
    ),
    re_path(
        r"^view/(?P<username>[\.\w]+)/$",
        login_required(messages_views.MessageDetailListView.as_view()),
        name="userena_umessages_detail",
    ),
    re_path(
        r"^remove/$", messages_views.message_remove, name="userena_umessages_remove"
    ),
    re_path(
        r"^unremove/$",
        messages_views.message_remove,
        {"undo": True},
        name="userena_umessages_unremove",
    ),
    re_path(
        r"^$",
        login_required(messages_views.MessageListView.as_view()),
        name="userena_umessages_list",
    ),
]



from django.test import TestCase
from django.core.management import call_command
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType

from userena.models import UserenaSignup
from userena.managers import ASSIGNED_PERMISSIONS
from userena import settings as userena_settings
from userena.utils import get_profile_model

from guardian.models import UserObjectPermission

import datetime

User = get_user_model()


class CleanExpiredTests(TestCase):
    user_info = {
        "username": "alice",
        "password": "swordfish",
        "email": "alice@example.com",
    }

    def test_clean_expired(self):
        """
        Test if ``clean_expired`` deletes all users which ``activation_key``
        is expired.

        """
        # Create an account which is expired.
        user = UserenaSignup.objects.create_user(**self.user_info)
        user.date_joined -= datetime.timedelta(
            days=userena_settings.USERENA_ACTIVATION_DAYS + 1
        )
        user.save()

        # There should be one account now
        User.objects.get(username=self.user_info["username"])

        # Clean it.
        call_command("clean_expired")

        self.assertEqual(
            User.objects.filter(username=self.user_info["username"]).count(), 0
        )


class CheckPermissionTests(TestCase):
    user_info = {
        "username": "alice",
        "password": "swordfish",
        "email": "alice@example.com",
    }

    def test_check_permissions(self):
        # Create a new account.
        user = UserenaSignup.objects.create_user(**self.user_info)
        user.save()

        # Remove all permissions
        UserObjectPermission.objects.filter(user=user).delete()
        self.assertEqual(UserObjectPermission.objects.filter(user=user).count(), 0)

        # Check it
        call_command("check_permissions")

        # User should have all permissions again
        user_permissions = UserObjectPermission.objects.filter(user=user).values_list(
            "permission__codename", flat=True
        )

        required_permissions = [
            "change_user",
            "delete_user",
            "change_profile",
            "view_profile",
        ]
        for perm in required_permissions:
            if perm not in user_permissions:
                self.fail()

        # Check it again should do nothing
        call_command("check_permissions", test=True)

    def test_incomplete_permissions(self):
        # Delete the neccesary permissions
        profile_model_obj = get_profile_model()
        content_type_profile = ContentType.objects.get_for_model(profile_model_obj)
        content_type_user = ContentType.objects.get_for_model(User)
        for model, perms in ASSIGNED_PERMISSIONS.items():
            if model == "profile":
                content_type = content_type_profile
            else:
                content_type = content_type_user
            for perm in perms:
                Permission.objects.get(name=perm[1], content_type=content_type).delete()

        # Check if they are they are back
        for model, perms in ASSIGNED_PERMISSIONS.items():
            if model == "profile":
                content_type = content_type_profile
            else:
                content_type = content_type_user
            for perm in perms:
                try:
                    perm = Permission.objects.get(
                        name=perm[1], content_type=content_type
                    )
                except Permission.DoesNotExist:
                    pass
                else:
                    self.fail("Found %s: " % perm)

        # Repair them
        call_command("check_permissions", test=True)

        # Check if they are they are back
        for model, perms in ASSIGNED_PERMISSIONS.items():
            if model == "profile":
                content_type = content_type_profile
            else:
                content_type = content_type_user
            for perm in perms:
                try:
                    perm = Permission.objects.get(
                        name=perm[1], content_type=content_type
                    )
                except Permission.DoesNotExist:
                    self.fail()

    def test_no_profile(self):
        """ Check for warning when there is no profile """
        # TODO: Dirty! Currently we check for the warning by getting a 100%
        # test coverage, meaning that it dit output some warning.
        user = UserenaSignup.objects.create_user(**self.user_info)

        # remove the profile of this user
        get_profile_model().objects.get(user=user).delete()

        # run the command to check for the warning.
        call_command("check_permissions", test=True)



# Userena settings file.
#
# Please consult the docs for more information about each setting.

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

gettext = lambda s: s


USERENA_SIGNIN_AFTER_SIGNUP = getattr(settings, "USERENA_SIGNIN_AFTER_SIGNUP", False)

USERENA_REDIRECT_ON_SIGNOUT = getattr(settings, "USERENA_REDIRECT_ON_SIGNOUT", None)

USERENA_SIGNIN_REDIRECT_URL = getattr(
    settings, "USERENA_SIGNIN_REDIRECT_URL", "/accounts/%(username)s/"
)

USERENA_ACTIVATION_REQUIRED = getattr(settings, "USERENA_ACTIVATION_REQUIRED", True)

USERENA_ACTIVATION_DAYS = getattr(settings, "USERENA_ACTIVATION_DAYS", 7)

USERENA_ACTIVATION_NOTIFY = getattr(settings, "USERENA_ACTIVATION_NOTIFY", True)

USERENA_ACTIVATION_NOTIFY_DAYS = getattr(settings, "USERENA_ACTIVATION_NOTIFY_DAYS", 5)

USERENA_ACTIVATION_RETRY = getattr(settings, "USERENA_ACTIVATION_RETRY", False)

USERENA_ACTIVATED = getattr(settings, "USERENA_ACTIVATED", "ALREADY_ACTIVATED")

USERENA_REMEMBER_ME_DAYS = getattr(
    settings, "USERENA_REMEMBER_ME_DAYS", (gettext("a month"), 30)
)

USERENA_FORBIDDEN_USERNAMES = getattr(
    settings,
    "USERENA_FORBIDDEN_USERNAMES",
    ("signup", "signout", "signin", "activate", "me", "password"),
)
DEFAULT_USERENA_USE_HTTPS = False

# NOTE: It is only for internal use. All those settings should be refactored to only defaults
#       as specified in #452
_USERENA_USE_HTTPS = getattr(settings, "USERENA_USE_HTTPS", DEFAULT_USERENA_USE_HTTPS)


USERENA_MUGSHOT_GRAVATAR = getattr(settings, "USERENA_MUGSHOT_GRAVATAR", True)

USERENA_MUGSHOT_GRAVATAR_SECURE = getattr(
    settings, "USERENA_MUGSHOT_GRAVATAR_SECURE", _USERENA_USE_HTTPS
)

USERENA_MUGSHOT_DEFAULT = getattr(settings, "USERENA_MUGSHOT_DEFAULT", "identicon")

USERENA_MUGSHOT_SIZE = getattr(settings, "USERENA_MUGSHOT_SIZE", 80)

USERENA_MUGSHOT_CROP_TYPE = getattr(settings, "USERENA_MUGSHOT_CROP_TYPE", "smart")

USERENA_MUGSHOT_PATH = getattr(settings, "USERENA_MUGSHOT_PATH", "mugshots/")

USERENA_DEFAULT_PRIVACY = getattr(settings, "USERENA_DEFAULT_PRIVACY", "registered")

USERENA_DISABLE_PROFILE_LIST = getattr(settings, "USERENA_DISABLE_PROFILE_LIST", False)

USERENA_DISABLE_SIGNUP = getattr(settings, "USERENA_DISABLE_SIGNUP", False)

USERENA_USE_MESSAGES = getattr(settings, "USERENA_USE_MESSAGES", True)

USERENA_LANGUAGE_FIELD = getattr(settings, "USERENA_LANGUAGE_FIELD", "language")

USERENA_WITHOUT_USERNAMES = getattr(settings, "USERENA_WITHOUT_USERNAMES", False)

USERENA_PROFILE_DETAIL_TEMPLATE = getattr(
    settings, "USERENA_PROFILE_DETAIL_TEMPLATE", "userena/profile_detail.html"
)

USERENA_PROFILE_LIST_TEMPLATE = getattr(
    settings, "USERENA_PROFILE_LIST_TEMPLATE", "userena/profile_list.html"
)

USERENA_HIDE_EMAIL = getattr(settings, "USERENA_HIDE_EMAIL", False)

USERENA_HTML_EMAIL = getattr(settings, "USERENA_HTML_EMAIL", False)

USERENA_USE_PLAIN_TEMPLATE = getattr(
    settings, "USERENA_USE_PLAIN_TEMPLATE", not USERENA_HTML_EMAIL
)

USERENA_REGISTER_PROFILE = getattr(settings, "USERENA_REGISTER_PROFILE", True)

USERENA_REGISTER_USER = getattr(settings, "USERENA_REGISTER_USER", True)

if hasattr(settings, "ANONYMOUS_USER_ID"):
    raise ImproperlyConfigured(
        "settings.ANONYMOUS_USER_ID is deprecated for settings.ANONYMOUS_USER_NAME. See https://django-guardian.readthedocs.io/en/stable/configuration.html"
    )

try:
    if settings.ANONYMOUS_USER_NAME == None:
        raise ImproperlyConfigured("settings.ANONYMOUS_USER_NAME must not be None.")
except AttributeError:
    raise ImproperlyConfigured(
        "ANONYMOUS_USER_NAME must be set in settings. See https://django-guardian.readthedocs.io/en/stable/configuration.html"
    )
